@@ -412,9 +412,9 @@ pub_glfs_set_volfile_server (struct glfs *fs, const char *transport,
                              const char *host, int port)
 {
         cmd_args_t            *cmd_args = NULL;
-        server_cmdline_t      *server = NULL;
-        server_cmdline_t      *tmp = NULL;
         int                    ret = -1;
+        char                  *server_host = NULL;
+        char                  *server_transport = NULL;
 
         if (!fs || !host) {
                 errno = EINVAL;
@@ -425,21 +425,10 @@ pub_glfs_set_volfile_server (struct glfs *fs, const char *transport,
         __GLFS_ENTRY_VALIDATE_FS (fs, invalid_fs);
 
         cmd_args = &fs->ctx->cmd_args;
-
         cmd_args->max_connect_attempts = 1;
 
-        server = GF_CALLOC (1, sizeof (server_cmdline_t),
-                            glfs_mt_server_cmdline_t);
-
-        if (!server) {
-                errno = ENOMEM;
-                goto out;
-        }
-
-        INIT_LIST_HEAD (&server->list);
-
-        server->volfile_server = gf_strdup (host);
-        if (!server->volfile_server) {
+        server_host = gf_strdup (host);
+        if (!server_host) {
                 errno = ENOMEM;
                 goto out;
         }
@@ -447,9 +436,10 @@ pub_glfs_set_volfile_server (struct glfs *fs, const char *transport,
         if (transport) {
                 /* volfile fetch support over tcp|unix only */
                 if (!strcmp(transport, "tcp") || !strcmp(transport, "unix")) {
-                        server->transport = gf_strdup (transport);
+                        server_transport = gf_strdup (transport);
                 } else if (!strcmp(transport, "rdma")) {
-                        server->transport = gf_strdup ("tcp");
+                        server_transport =
+                                gf_strdup (GF_DEFAULT_VOLFILE_TRANSPORT);
                         gf_msg ("glfs", GF_LOG_WARNING, EINVAL,
                                 API_MSG_INVALID_ENTRY,
                                 "transport RDMA is deprecated, "
@@ -460,55 +450,39 @@ pub_glfs_set_volfile_server (struct glfs *fs, const char *transport,
                                 "transport %s is not supported, "
                                 "possible values tcp|unix",
                                 transport);
-                        ret = -1;
                         goto out;
                 }
         } else {
-                server->transport = gf_strdup (GF_DEFAULT_VOLFILE_TRANSPORT);
+                server_transport = gf_strdup (GF_DEFAULT_VOLFILE_TRANSPORT);
         }
 
-        if (!server->transport) {
+        if (!server_transport) {
                 errno = ENOMEM;
                 goto out;
         }
 
-        if (strcmp(server->transport, "unix")) {
-                if (port) {
-                        server->port = port;
-                } else {
-                        server->port = GF_DEFAULT_BASE_PORT;
-                }
-        } else {
-                server->port = 0;
+        if (!port) {
+                port = GF_DEFAULT_BASE_PORT;
         }
 
-        if (!cmd_args->volfile_server) {
-                cmd_args->volfile_server = server->volfile_server;
-                cmd_args->volfile_server_transport = server->transport;
-                cmd_args->volfile_server_port = server->port;
-                cmd_args->curr_server = server;
+        if (!strcmp(server_transport, "unix")) {
+                port = 0;
         }
 
-        list_for_each_entry(tmp, &cmd_args->volfile_servers, list) {
-                if ((!strcmp(tmp->volfile_server, server->volfile_server) &&
-                     !strcmp(tmp->transport, server->transport) &&
-                     (tmp->port == server->port))) {
-                        errno = EEXIST;
-                        ret = -1;
-                        goto out;
-                }
+        ret = gf_set_volfile_server_common(cmd_args, server_host,
+                                           server_transport, port);
+        if (ret) {
+                gf_log ("glfs", GF_LOG_ERROR,
+                        "failed to set volfile server: %s", strerror (errno));
         }
 
-        list_add_tail (&server->list, &cmd_args->volfile_servers);
-
-        ret = 0;
 out:
-        if (ret == -1) {
-                if (server) {
-                        GF_FREE (server->volfile_server);
-                        GF_FREE (server->transport);
-                        GF_FREE (server);
-                }
+        if (server_host) {
+                GF_FREE (server_host);
+        }
+
+        if (server_transport) {
+                GF_FREE (server_transport);
         }
 
         __GLFS_EXIT_FS;
@@ -604,7 +578,7 @@ pub_glfs_setfsleaseid (glfs_leaseid_t leaseid)
 
         GF_VALIDATE_OR_GOTO (THIS->name, leaseid, out);
 
-        gleaseid = glusterfs_leaseid_buf_get();
+        gleaseid = gf_leaseid_get();
         if (gleaseid) {
                 memcpy (gleaseid, leaseid, LEASE_ID_SIZE);
                 ret = 0;
@@ -615,6 +589,87 @@ out:
 
 GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_setfsleaseid, 4.0.0);
 
+int
+get_fop_attr_glfd (dict_t **fop_attr, struct glfs_fd *glfd)
+{
+        char *leaseid            = NULL;
+        int   ret                = 0;
+        gf_boolean_t dict_create = _gf_false;
+
+        leaseid = GF_CALLOC (1, LEASE_ID_SIZE, gf_common_mt_char);
+        GF_CHECK_ALLOC_AND_LOG("gfapi", leaseid, ret, "lease id alloc failed", out);
+        memcpy (leaseid, glfd->lease_id, LEASE_ID_SIZE);
+        if (*fop_attr == NULL) {
+                *fop_attr = dict_new ();
+                dict_create = _gf_true;
+        }
+        GF_CHECK_ALLOC_AND_LOG("gfapi", *fop_attr, ret, "dict_new failed", out);
+        ret = dict_set_static_bin (*fop_attr, "lease-id", leaseid, LEASE_ID_SIZE);
+out:
+        if (ret) {
+                GF_FREE (leaseid);
+                if (dict_create) {
+                        if (*fop_attr)
+                                dict_unref (*fop_attr);
+                        *fop_attr = NULL;
+                }
+        }
+        return ret;
+}
+
+int
+set_fop_attr_glfd (struct glfs_fd *glfd)
+{
+        char     *lease_id = NULL;
+        int       ret      = -1;
+
+        lease_id = gf_existing_leaseid ();
+        if (lease_id) {
+                memcpy (glfd->lease_id, lease_id, LEASE_ID_SIZE);
+                ret = 0;
+        }
+        return ret;
+}
+
+int
+get_fop_attr_thrd_key (dict_t **fop_attr)
+{
+        char     *lease_id       = NULL;
+        int       ret            = 0;
+        gf_boolean_t dict_create = _gf_false;
+
+        lease_id = gf_existing_leaseid ();
+        if (lease_id) {
+                if (*fop_attr == NULL) {
+                        *fop_attr = dict_new ();
+                        dict_create = _gf_true;
+                }
+                GF_CHECK_ALLOC_AND_LOG("gfapi", *fop_attr, ret, "dict_new failed", out);
+                ret = dict_set_bin (*fop_attr, "lease-id", gf_strdup (lease_id),
+                                    LEASE_ID_SIZE);
+        }
+
+out:
+        if (ret && dict_create) {
+                if (*fop_attr)
+                        dict_unref (*fop_attr);
+                *fop_attr = NULL;
+        }
+        return ret;
+}
+
+void
+unset_fop_attr (dict_t **fop_attr)
+{
+        char     *lease_id = NULL;
+        lease_id = gf_existing_leaseid ();
+        if (lease_id)
+                memset (lease_id, 0, LEASE_ID_SIZE);
+        if (*fop_attr) {
+                dict_unref (*fop_attr);
+                *fop_attr = NULL;
+        }
+}
 struct glfs *
 pub_glfs_from_glfd (struct glfs_fd *glfd)
 {
@@ -823,6 +878,7 @@ pub_glfs_new (const char *volname)
                 goto out;
 
         fs->ctx = ctx;
+        fs->ctx->process_mode = GF_CLIENT_PROCESS;
 
         ret = glfs_set_logging (fs, "/dev/null", 0);
         if (ret)
@@ -1511,6 +1567,22 @@ pub_glfs_upcall_inode_get_oldpstat (struct glfs_upcall_inode *arg)
 GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_upcall_inode_get_oldpstat, 3.7.16);
 
 
+/*struct glfs_object*
+pub_glfs_upcall_lease_get_object (struct glfs_upcall_recall_inode *arg)
+{
+        return arg->object;
+}
+
+GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_upcall_lease_get_object, 4.0.0);
+*/
+
+uint32_t
+pub_glfs_upcall_lease_get_lease_type (struct glfs_upcall_lease *arg)
+{
+        return arg->lease_type;
+}
+GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_upcall_lease_get_lease_type, 4.0.0);
+
 /* definitions of the GLFS_SYSRQ_* chars are in glfs.h */
 static struct glfs_sysrq_help {
         char  sysrq;
@@ -1569,8 +1641,8 @@ out:
 GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_sysrq, 3.10.0);
 
 int
-glfs_upcall_register (struct glfs *fs, uint32_t event_list,
-                      glfs_upcall_cbk cbk, void *data)
+pub_glfs_upcall_register (struct glfs *fs, uint32_t event_list,
+                          glfs_upcall_cbk cbk, void *data)
 {
         int ret = 0;
 
@@ -1603,8 +1675,10 @@ glfs_upcall_register (struct glfs *fs, uint32_t event_list,
                          */
                         fs->upcall_events |=  GF_UPCALL_CACHE_INVALIDATION;
                         ret |= GF_UPCALL_CACHE_INVALIDATION;
+                } else if (event_list & GLFS_EVENT_INODE_INVALIDATE) {
+                        fs->upcall_events |=  GF_UPCALL_RECALL_LEASE;
+                        ret |= GF_UPCALL_RECALL_LEASE;
                 }
-
                 /* Override cbk function if existing */
                 fs->up_cbk = cbk;
                 fs->up_data = data;
@@ -1618,9 +1692,11 @@ out:
 invalid_fs:
         return ret;
 }
+
 GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_upcall_register, 3.13.0);
 
-int glfs_upcall_unregister (struct glfs *fs, uint32_t event_list)
+int
+pub_glfs_upcall_unregister (struct glfs *fs, uint32_t event_list)
 {
         int ret = 0;
         /* list of supported upcall events */
@@ -1663,4 +1739,5 @@ out:
 invalid_fs:
         return ret;
 }
+
 GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_upcall_unregister, 3.13.0);
