@@ -1,3 +1,4 @@
+#!/usr/bin/python2
 #
 # Copyright (c) 2016 Red Hat, Inc. <http://www.redhat.com>
 # This file is part of GlusterFS.
@@ -8,4 +9,4 @@
 # cases as published by the Free Software Foundation.
 #
 
-GLUSTERFS_LIBEXECDIR = '${exec_prefix}/libexec/glusterfs'
+GLUSTERFS_LIBEXECDIR = '/usr/local/libexec/glusterfs'
