@@ -88,7 +88,7 @@ def takeSnap(volname="", snapname=""):
 def doJob(name, lockFile, jobFunc, volname):
     success = True
     try:
-        f = os.open(lockFile, os.O_RDWR | os.O_NONBLOCK)
+        f = os.open(lockFile, os.O_CREAT | os.O_RDWR | os.O_NONBLOCK)
         try:
             fcntl.flock(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
             mtime = os.path.getmtime(lockFile)
@@ -105,10 +105,10 @@ def doJob(name, lockFile, jobFunc, volname):
             else:
                 log.info("Job %s has been processed already", name)
             fcntl.flock(f, fcntl.LOCK_UN)
-        except IOError as (errno, strerror):
+        except (OSError, IOError) as (errno, strerror):
             log.info("Job %s is being processed by another agent", name)
         os.close(f)
-    except IOError as (errno, strerror):
+    except (OSError, IOError) as (errno, strerror):
         log.debug("Failed to open lock file %s : %s", lockFile, strerror)
         log.error("Failed to process job %s", name)
         success = False
