@@ -101,6 +101,12 @@ enum argp_option_keys {
         ARGP_PROCESS_NAME_KEY             = 179,
         ARGP_FUSE_EVENT_HISTORY_KEY       = 180,
         ARGP_THIN_CLIENT_KEY              = 181,
+        ARGP_READER_THREAD_COUNT_KEY      = 182,
+        ARGP_PRINT_XLATORDIR_KEY          = 183,
+        ARGP_PRINT_STATEDUMPDIR_KEY       = 184,
+        ARGP_PRINT_LOGDIR_KEY             = 185,
+        ARGP_KERNEL_WRITEBACK_CACHE_KEY   = 186,
+        ARGP_ATTR_TIMES_GRANULARITY_KEY   = 187,
 };
 
 struct _gfd_vol_top_priv {
@@ -124,6 +130,8 @@ int glusterfs_volume_top_write_perf (uint32_t blk_size, uint32_t blk_count,
 int glusterfs_volume_top_read_perf (uint32_t blk_size, uint32_t blk_count,
                                     char *brick_path, double *throughput,
                                     double *time);
+void
+glusterfs_autoscale_threads (glusterfs_ctx_t *ctx, int incr, xlator_t *this);
 
 extern glusterfs_ctx_t *glusterfsd_ctx;
 #endif /* __GLUSTERFSD_H__ */
