@@ -18,10 +18,11 @@
 #include "list.h"
 
 typedef struct _call_stub {
-	struct list_head list;
-	char wind;
-	call_frame_t *frame;
-	glusterfs_fop_t fop;
+        struct list_head list;
+        char wind;
+        call_frame_t *frame;
+        glusterfs_fop_t fop;
+        gf_boolean_t poison;
         struct mem_pool *stub_mem_pool; /* pointer to stub mempool in ctx_t */
         uint32_t jnl_meta_len;
         uint32_t jnl_data_len;
@@ -77,6 +78,9 @@ typedef struct _call_stub {
                 fop_lease_t lease;
                 fop_getactivelk_t getactivelk;
 	        fop_setactivelk_t setactivelk;
+                fop_put_t put;
+                fop_icreate_t icreate;
+                fop_namelink_t namelink;
         } fn;
 
 	union {
@@ -129,6 +133,9 @@ typedef struct _call_stub {
                 fop_lease_cbk_t lease;
                 fop_getactivelk_cbk_t getactivelk;
                 fop_setactivelk_cbk_t setactivelk;
+                fop_put_cbk_t put;
+                fop_icreate_cbk_t icreate;
+                fop_namelink_cbk_t namelink;
 	} fn_cbk;
 
         default_args_t args;
@@ -762,6 +769,38 @@ call_stub_t *
 fop_setactivelk_cbk_stub (call_frame_t *frame, fop_setactivelk_cbk_t fn,
                            int32_t op_ret, int32_t op_errno, dict_t *xdata);
 
+call_stub_t *
+fop_put_stub (call_frame_t *frame, fop_put_t fn, loc_t *loc,
+              mode_t mode, mode_t umask, uint32_t flags,
+              struct iovec *vector, int32_t count, off_t offset,
+              struct iobref *iobref, dict_t *xattr, dict_t *xdata);
+
+call_stub_t *
+fop_put_cbk_stub (call_frame_t *frame, fop_put_cbk_t fn, int32_t op_ret,
+                  int32_t op_errno, inode_t *inode, struct iatt *buf,
+                  struct iatt *preparent, struct iatt *postparent,
+                  dict_t *xdata);
+
+call_stub_t *
+fop_icreate_stub (call_frame_t *frame, fop_icreate_t fn,
+                  loc_t *loc, mode_t mode, dict_t *xdata);
+
+call_stub_t *
+fop_namelink_stub (call_frame_t *frame,
+                   fop_namelink_t fn, loc_t *loc, dict_t *xdata);
+
+call_stub_t *
+fop_icreate_cbk_stub (call_frame_t *frame,
+                      fop_icreate_cbk_t fn,
+                      int32_t op_ret, int32_t op_errno,
+                      inode_t *inode, struct iatt *buf, dict_t *xdata);
+
+call_stub_t *
+fop_namelink_cbk_stub (call_frame_t *frame,
+                       fop_namelink_cbk_t fn,
+                       int32_t op_ret, int32_t op_errno,
+                       struct iatt *prebuf, struct iatt *postbuf, dict_t *xdata);
+
 void call_resume (call_stub_t *stub);
 void call_resume_keep_stub (call_stub_t *stub);
 void call_stub_destroy (call_stub_t *stub);
