@@ -3093,7 +3093,8 @@ dict_unserialize (char *orig_buf, int32_t size, dict_t **fill)
                 vallen = ntoh32 (hostord);
                 buf += DICT_DATA_HDR_VAL_LEN;
 
-                if ((buf + keylen) > (orig_buf + size)) {
+                if ((keylen < 0) || (vallen < 0) ||
+                    (buf + keylen) > (orig_buf + size)) {
                         gf_msg_callingfn ("dict", GF_LOG_ERROR, 0,
                                           LG_MSG_UNDERSIZED_BUF,
                                           "undersized buffer passed. "
