@@ -60,13 +60,21 @@ typedef struct _data_pair data_pair_t;
                                                                         \
         } while (0)
 
+#define dict_foreach_inline(d, c) for (c = d->members_list; c; c = c->next)
+
+#define DICT_KEY_VALUE_MAX_SIZE                     1048576
+#define DICT_MAX_FLAGS                              256
+#define DICT_FLAG_SET          1
+#define DICT_FLAG_CLEAR        0
+
 struct _data {
         unsigned char  is_static:1;
         unsigned char  is_const:1;
         int32_t        len;
         char          *data;
-        int32_t        refcount;
+        gf_atomic_t    refcount;
         gf_lock_t      lock;
+        gf_dict_data_type_t data_type;
 };
 
 struct _data_pair {
@@ -82,7 +90,7 @@ struct _dict {
         unsigned char   is_static:1;
         int32_t         hash_size;
         int32_t         count;
-        int32_t         refcount;
+        gf_atomic_t     refcount;
         data_pair_t   **members;
         data_pair_t    *members_list;
         char           *extra_free;
@@ -91,7 +99,7 @@ struct _dict {
         data_pair_t    *members_internal;
         data_pair_t     free_pair;
         gf_boolean_t    free_pair_in_use;
-        uint32_t        max_count;
+        uint64_t        max_count;
 };
 
 typedef gf_boolean_t (*dict_match_t) (dict_t *d, char *k, data_t *v,
@@ -157,8 +165,8 @@ data_t *data_from_uint16 (uint16_t value);
 char *data_to_str (data_t *data);
 void *data_to_bin (data_t *data);
 void *data_to_ptr (data_t *data);
-
 data_t * data_copy (data_t *old);
+
 int dict_foreach (dict_t *this,
                   int (*fn)(dict_t *this,
                             char *key,
@@ -222,6 +230,10 @@ GF_MUST_CHECK int dict_set_uint32 (dict_t *this, char *key, uint32_t val);
 GF_MUST_CHECK int dict_get_uint64 (dict_t *this, char *key, uint64_t *val);
 GF_MUST_CHECK int dict_set_uint64 (dict_t *this, char *key, uint64_t val);
 
+GF_MUST_CHECK int dict_check_flag (dict_t *this, char *key, int flag);
+GF_MUST_CHECK int dict_set_flag (dict_t *this, char *key, int flag);
+GF_MUST_CHECK int dict_clear_flag (dict_t *this, char *key, int flag);
+
 GF_MUST_CHECK int dict_get_double (dict_t *this, char *key, double *val);
 GF_MUST_CHECK int dict_set_double (dict_t *this, char *key, double val);
 
@@ -235,6 +247,7 @@ GF_MUST_CHECK int dict_get_bin (dict_t *this, char *key, void **ptr);
 GF_MUST_CHECK int dict_set_bin (dict_t *this, char *key, void *ptr, size_t size);
 GF_MUST_CHECK int dict_set_static_bin (dict_t *this, char *key, void *ptr, size_t size);
 
+GF_MUST_CHECK int dict_set_option (dict_t *this, char *key, char *str);
 GF_MUST_CHECK int dict_set_str (dict_t *this, char *key, char *str);
 GF_MUST_CHECK int dict_set_dynstr (dict_t *this, char *key, char *str);
 GF_MUST_CHECK int dict_set_dynstr_with_alloc (dict_t *this, char *key, const char *str);
@@ -245,6 +258,15 @@ GF_MUST_CHECK int dict_get_str_boolean (dict_t *this, char *key, int default_val
 GF_MUST_CHECK int dict_rename_key (dict_t *this, char *key, char *replace_key);
 GF_MUST_CHECK int dict_serialize_value_with_delim (dict_t *this, char *buf, int32_t *serz_len,
                                                     char delimiter);
+
+GF_MUST_CHECK int dict_set_gfuuid (dict_t *this, char *key, uuid_t uuid,
+                                   bool is_static);
+GF_MUST_CHECK int dict_get_gfuuid (dict_t *this, char *key, uuid_t *uuid);
+
+GF_MUST_CHECK int dict_set_iatt (dict_t *this, char *key, struct iatt *iatt,
+                                 bool is_static);
+GF_MUST_CHECK int dict_get_iatt (dict_t *this, char *key, struct iatt *iatt);
+
 void
 dict_dump_to_statedump (dict_t *dict, char *dict_name, char *domain);
 
