@@ -14,10 +14,16 @@
 #define GF_DEFAULT_BASE_PORT 24007
 #define GF_DEFAULT_VOLFILE_TRANSPORT "tcp"
 
+#define GF_GLOBAL_XLATOR_NAME "global"
 #define GD_OP_VERSION_KEY     "operating-version"
 #define GD_MIN_OP_VERSION_KEY "minimum-operating-version"
 #define GD_MAX_OP_VERSION_KEY "maximum-operating-version"
 
+#define GF_PROTECT_FROM_EXTERNAL_WRITES "trusted.glusterfs.protect.writes"
+#define GF_AVOID_OVERWRITE "glusterfs.avoid.overwrite"
+#define GF_CLEAN_WRITE_PROTECTION "glusterfs.clean.writexattr"
+
+
 /* Gluster versions - OP-VERSION mapping
  *
  * 3.3.x                - 1
@@ -38,7 +44,7 @@
  */
 #define GD_OP_VERSION_MIN  1 /* MIN is the fresh start op-version, mostly
                                 should not change */
-#define GD_OP_VERSION_MAX  GD_OP_VERSION_3_13_2 /* MAX VERSION is the maximum
+#define GD_OP_VERSION_MAX  GD_OP_VERSION_4_1_0 /* MAX VERSION is the maximum
                                                   count in VME table, should
                                                   keep changing with
                                                   introduction of newer
@@ -98,10 +104,14 @@
 
 #define GD_OP_VERSION_3_13_2   31302 /* Op-version for GlusterFS 3.13.2 */
 
+#define GD_OP_VERSION_4_0_0    40000 /* Op-version for GlusterFS 4.0.0 */
+
+#define GD_OP_VERSION_4_1_0    40100 /* Op-version for GlusterFS 4.1.0 */
 
 #define GD_OP_VER_PERSISTENT_AFR_XATTRS GD_OP_VERSION_3_6_0
 
 #include "xlator.h"
+#include "options.h"
 
 /* THIS */
 #define THIS (*__glusterfs_this_location())
@@ -111,6 +121,9 @@ xlator_t **__glusterfs_this_location (void);
 xlator_t *glusterfs_this_get (void);
 int glusterfs_this_set (xlator_t *);
 
+extern xlator_t global_xlator;
+extern struct volume_options global_xl_options[];
+
 /* syncopctx */
 void *syncopctx_getctx (void);
 int syncopctx_setctx (void *ctx);
@@ -125,6 +138,7 @@ char *glusterfs_uuid_buf_get (void);
 char *glusterfs_lkowner_buf_get (void);
 /* leaseid buf */
 char *glusterfs_leaseid_buf_get (void);
+char *glusterfs_leaseid_exist (void);
 
 /* init */
 int glusterfs_globals_init (glusterfs_ctx_t *ctx);
