@@ -32,6 +32,7 @@ typedef struct call_pool call_pool_t;
 #include "lkowner.h"
 #include "client_t.h"
 #include "libglusterfs-messages.h"
+#include "timespec.h"
 
 #define NFS_PID 1
 #define LOW_PRIO_PROC_PID -1
@@ -55,6 +56,7 @@ struct call_pool {
                 } all_stacks;
         };
         int64_t                     cnt;
+        gf_atomic_t                 total_count;
         gf_lock_t                   lock;
         struct mem_pool             *frame_mem_pool;
         struct mem_pool             *stack_mem_pool;
@@ -72,15 +74,22 @@ struct _call_frame {
         void         *cookie;      /* unique cookie */
         gf_boolean_t  complete;
 
-        glusterfs_fop_t op;
-        struct timeval begin;      /* when this frame was created */
-        struct timeval end;        /* when this frame completed */
+        glusterfs_fop_t  op;
+        struct timespec  begin;      /* when this frame was created */
+        struct timespec  end;        /* when this frame completed */
         const char      *wind_from;
         const char      *wind_to;
         const char      *unwind_from;
         const char      *unwind_to;
 };
 
+struct _ns_info {
+        uint32_t     hash;  /* Hash of the namespace from SuperFastHash */
+        gf_boolean_t found; /* Set to true if we found a namespace */
+};
+
+typedef struct _ns_info ns_info_t;
+
 #define SMALL_GROUP_COUNT 128
 
 struct _call_stack {
@@ -112,12 +121,27 @@ struct _call_stack {
 
         int32_t                       op;
         int8_t                        type;
-        struct timeval                tv;
+        struct timespec               tv;
         xlator_t                     *err_xl;
         int32_t                       error;
+
+        uint32_t flags; /* use it wisely, think of it as a mechanism to
+                           send information over the wire too */
+        struct timespec ctime; /* timestamp, most probably set at
+                                  creation of stack. */
+
+        ns_info_t                     ns_info;
 };
 
 
+/* call_stack flags field users */
+#define MDATA_CTIME (1<<0)
+#define MDATA_MTIME (1<<1)
+#define MDATA_ATIME (1<<2)
+#define MDATA_PAR_CTIME (1<<3)
+#define MDATA_PAR_MTIME (1<<4)
+#define MDATA_PAR_ATIME (1<<5)
+
 #define frame_set_uid_gid(frm, u, g)            \
         do {                                    \
                 if (frm) {                      \
@@ -129,18 +153,17 @@ struct _call_stack {
 
 
 struct xlator_fops;
+void gf_update_latency (call_frame_t *frame);
 
-void
-gf_latency_begin (call_frame_t *frame, void *fn);
-
-void
-gf_latency_end (call_frame_t *frame);
 
 static inline void
 FRAME_DESTROY (call_frame_t *frame)
 {
         void *local = NULL;
 
+        if (frame->root->ctx->measure_latency)
+                gf_update_latency (frame);
+
         list_del_init (&frame->frames);
         if (frame->local) {
                 local = frame->local;
@@ -225,12 +248,25 @@ STACK_RESET (call_stack_t *stack)
         } while (0);                                                   \
 
 
+/* NOTE: make sure to keep this as an macro, mainly because, we need 'fn'
+   field here to be the proper fn ptr, so its address is valid entry in
+   'xlator_fops' struct.
+   To understand this, check the `xlator.h:struct xlator_fops`, and then
+   see a STACK_WIND call, which generally calls `subvol->fops->fop`, so
+   the address offset should give the index */
+
+/* +1 is required as 0 means NULL fop, and we don't have a variable for it */
+#define get_fop_index_from_fn(xl, fn)                                   \
+        (1 + (((long)&(fn) - (long)&((xl)->fops->stat)) / sizeof (void *)))
+
+
 /* make a call without switching frames */
 #define STACK_WIND_TAIL(frame, obj, fn, params ...)                     \
         do {                                                            \
                 xlator_t     *old_THIS = NULL;                          \
                 xlator_t     *next_xl = obj;                            \
                 typeof(fn)    next_xl_fn = fn;                          \
+                int opn = get_fop_index_from_fn((next_xl), (fn));       \
                                                                         \
                 frame->this = next_xl;                                  \
                 frame->wind_to = #fn;                                   \
@@ -241,6 +277,16 @@ STACK_RESET (call_stack_t *stack)
                               "winding from %s to %s",                  \
                               frame->root, old_THIS->name,              \
                               THIS->name);                              \
+                /* Need to capture counts at leaf node */               \
+                if (!next_xl->pass_through && !next_xl->children) {     \
+                        GF_ATOMIC_INC (next_xl->stats.total.metrics[opn].fop); \
+                        GF_ATOMIC_INC (next_xl->stats.interval.metrics[opn].fop); \
+                        GF_ATOMIC_INC (next_xl->stats.total.count);     \
+                        GF_ATOMIC_INC (next_xl->stats.interval.count);  \
+                }                                                       \
+                if (next_xl->pass_through) {                            \
+                        next_xl_fn = (void *)*((&next_xl->pass_through_fops->stat) + (opn - 1)); \
+                }                                                       \
                 next_xl_fn (frame, next_xl, params);                    \
                 THIS = old_THIS;                                        \
         } while (0)
@@ -263,6 +309,7 @@ STACK_RESET (call_stack_t *stack)
         do {                                                            \
                 call_frame_t *_new = NULL;                              \
                 xlator_t     *old_THIS = NULL;                          \
+                typeof(fn)    next_xl_fn = fn;                          \
                                                                         \
                 _new = mem_get0 (frame->root->pool->frame_mem_pool);    \
                 if (!_new) {                                            \
@@ -295,8 +342,17 @@ STACK_RESET (call_stack_t *stack)
                               frame->root, old_THIS->name,              \
                               THIS->name);                              \
                 if (obj->ctx->measure_latency)                          \
-                        gf_latency_begin (_new, fn);                    \
-                fn (_new, obj, params);                                 \
+                        timespec_now (&_new->begin);                    \
+                _new->op = get_fop_index_from_fn ((_new->this), (fn));  \
+                if (!obj->pass_through) {                               \
+                        GF_ATOMIC_INC (obj->stats.total.metrics[_new->op].fop); \
+                        GF_ATOMIC_INC (obj->stats.interval.metrics[_new->op].fop); \
+                        GF_ATOMIC_INC (obj->stats.total.count);         \
+                        GF_ATOMIC_INC (obj->stats.interval.count);      \
+                } else {                                                \
+                        next_xl_fn = (void *)*((&obj->pass_through_fops->stat) + (_new->op - 1)); \
+                }                                                       \
+                next_xl_fn (_new, obj, params);                         \
                 THIS = old_THIS;                                        \
         } while (0)
 
@@ -304,9 +360,9 @@ STACK_RESET (call_stack_t *stack)
 #define STACK_UNWIND STACK_UNWIND_STRICT
 
 /* return from function in type-safe way */
-#define STACK_UNWIND_STRICT(op, frame, op_ret, op_errno, params ...)    \
+#define STACK_UNWIND_STRICT(fop, frame, op_ret, op_errno, params ...)   \
         do {                                                            \
-                fop_##op##_cbk_t      fn = NULL;                        \
+                fop_##fop##_cbk_t      fn = NULL;                       \
                 call_frame_t *_parent = NULL;                           \
                 xlator_t     *old_THIS = NULL;                          \
                                                                         \
@@ -329,7 +385,7 @@ STACK_RESET (call_stack_t *stack)
                                       frame->root, THIS->name,          \
                                       (int32_t)(op_ret));               \
                 }                                                       \
-                fn = (fop_##op##_cbk_t )frame->ret;                     \
+                fn = (fop_##fop##_cbk_t)frame->ret;                     \
                 _parent = frame->parent;                                \
                 LOCK(&frame->root->stack_lock);                         \
                 {                                                       \
@@ -348,34 +404,37 @@ STACK_RESET (call_stack_t *stack)
                 THIS = _parent->this;                                   \
                 frame->complete = _gf_true;                             \
                 frame->unwind_from = __FUNCTION__;                      \
-                if (frame->this->ctx->measure_latency)                  \
-                        gf_latency_end (frame);                         \
+                if (frame->this->ctx->measure_latency) {                \
+                        timespec_now (&frame->end);                     \
+                        /* required for top most xlator */              \
+                        if (_parent->ret == NULL)                       \
+                                timespec_now (&_parent->end);           \
+                }                                                       \
+                if (op_ret < 0) {                                       \
+                        GF_ATOMIC_INC (THIS->stats.total.metrics[frame->op].cbk); \
+                        GF_ATOMIC_INC (THIS->stats.interval.metrics[frame->op].cbk); \
+                }                                                       \
                 fn (_parent, frame->cookie, _parent->this, op_ret,      \
                     op_errno, params);                                  \
                 THIS = old_THIS;                                        \
         } while (0)
 
 
-static void
-call_stack_set_groups (call_stack_t *stack, int ngrps, gid_t *groupbuf)
-{
-        stack->groups = groupbuf;
-        stack->ngrps = ngrps;
-}
-
 static inline int
 call_stack_alloc_groups (call_stack_t *stack, int ngrps)
 {
 	if (ngrps <= SMALL_GROUP_COUNT) {
-		call_stack_set_groups (stack, ngrps, stack->groups_small);
+		stack->groups = stack->groups_small;
 	} else {
 		stack->groups_large = GF_CALLOC (ngrps, sizeof (gid_t),
 						 gf_common_mt_groups_t);
 		if (!stack->groups_large)
 			return -1;
-		call_stack_set_groups (stack, ngrps, stack->groups_large);
+		stack->groups = stack->groups_large;
 	}
 
+	stack->ngrps = ngrps;
+
 	return 0;
 }
 
@@ -430,6 +489,8 @@ copy_frame (call_frame_t *frame)
         newstack->pid = oldstack->pid;
         newstack->op  = oldstack->op;
         newstack->type = oldstack->type;
+        newstack->ctime = oldstack->ctime;
+        newstack->flags = oldstack->flags;
 	if (call_stack_alloc_groups (newstack, oldstack->ngrps) != 0) {
 		mem_put (newstack);
 		return NULL;
@@ -454,10 +515,7 @@ copy_frame (call_frame_t *frame)
         newstack->ctx = oldstack->ctx;
 
         if (newstack->ctx->measure_latency) {
-                if (gettimeofday (&newstack->tv, NULL) == -1)
-                        gf_msg ("stack", GF_LOG_ERROR, errno,
-                                LG_MSG_GETTIMEOFDAY_FAILED,
-                                "gettimeofday () failed.");
+                timespec_now (&newstack->tv);
                 memcpy (&newframe->begin, &newstack->tv,
                         sizeof (newstack->tv));
         }
@@ -471,10 +529,12 @@ copy_frame (call_frame_t *frame)
                 newstack->pool->cnt++;
         }
         UNLOCK (&oldstack->pool->lock);
+        GF_ATOMIC_INC (newstack->pool->total_count);
 
         return newframe;
 }
 
+void call_stack_set_groups (call_stack_t *stack, int ngrps, gid_t **groupbuf_p);
 void gf_proc_dump_pending_frames(call_pool_t *call_pool);
 void gf_proc_dump_pending_frames_to_dict (call_pool_t *call_pool,
                                           dict_t *dict);
