@@ -1087,30 +1087,52 @@ synclock_unlock (synclock_t *lock)
 int
 syncbarrier_init (struct syncbarrier *barrier)
 {
+        int ret = 0;
 	if (!barrier) {
 		errno = EINVAL;
 		return -1;
 	}
 
-	pthread_cond_init (&barrier->cond, 0);
+	ret = pthread_cond_init (&barrier->cond, 0);
+        if (ret) {
+                errno = ret;
+                return -1;
+        }
 	barrier->count = 0;
         barrier->waitfor = 0;
 	INIT_LIST_HEAD (&barrier->waitq);
 
-	return pthread_mutex_init (&barrier->guard, 0);
+	ret = pthread_mutex_init (&barrier->guard, 0);
+        if (ret) {
+                (void)pthread_cond_destroy (&barrier->cond);
+                errno = ret;
+                return -1;
+        }
+        barrier->initialized = _gf_true;
+        return 0;
 }
 
 
 int
 syncbarrier_destroy (struct syncbarrier *barrier)
 {
+        int ret = 0;
+        int ret1 = 0;
 	if (!barrier) {
 		errno = EINVAL;
 		return -1;
 	}
 
-	pthread_cond_destroy (&barrier->cond);
-	return pthread_mutex_destroy (&barrier->guard);
+        if (barrier->initialized) {
+                ret = pthread_cond_destroy (&barrier->cond);
+                ret1 = pthread_mutex_destroy (&barrier->guard);
+                barrier->initialized = _gf_false;
+        }
+        if (ret || ret1) {
+                errno = ret?ret:ret1;
+                return -1;
+        }
+        return 0;
 }
 
 
@@ -3068,33 +3090,30 @@ syncop_inodelk (xlator_t *subvol, const char *volume, loc_t *loc, int32_t cmd,
 }
 
 int32_t
-syncop_xattrop_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                    int32_t op_ret, int32_t op_errno, dict_t *dict,
-                    dict_t *xdata)
+syncop_entrylk_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+		    int32_t op_ret, int32_t op_errno, dict_t *xdata)
 {
         struct syncargs *args = NULL;
 
         args = cookie;
-
-        args->op_ret   = op_ret;
+        args->op_ret = op_ret;
         args->op_errno = op_errno;
         if (xdata)
-                args->xdata  = dict_ref (xdata);
+                args->xdata = dict_ref (xdata);
 
         __wake (args);
-
         return 0;
-
 }
 
 int
-syncop_xattrop (xlator_t *subvol, loc_t *loc, gf_xattrop_flags_t flags,
-                dict_t *dict, dict_t *xdata_in, dict_t **xdata_out)
+syncop_entrylk (xlator_t *subvol, const char *volume, loc_t *loc,
+                const char *basename, entrylk_cmd cmd, entrylk_type type,
+                dict_t *xdata_in, dict_t **xdata_out)
 {
         struct syncargs args = {0, };
 
-        SYNCOP (subvol, (&args), syncop_xattrop_cbk, subvol->fops->xattrop,
-                loc, flags, dict, xdata_in);
+        SYNCOP (subvol, (&args), syncop_entrylk_cbk, subvol->fops->entrylk,
+                volume, loc, basename, cmd, type, xdata_in);
 
         if (xdata_out)
                 *xdata_out = args.xdata;
@@ -3107,9 +3126,58 @@ syncop_xattrop (xlator_t *subvol, loc_t *loc, gf_xattrop_flags_t flags,
         return args.op_ret;
 }
 
+int32_t
+syncop_xattrop_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                    int32_t op_ret, int32_t op_errno, dict_t *dict,
+                    dict_t *xdata)
+{
+        struct syncargs *args = NULL;
+
+        args = cookie;
+
+        args->op_ret   = op_ret;
+        args->op_errno = op_errno;
+        if (xdata)
+                args->xdata  = dict_ref (xdata);
+        if (dict)
+                args->dict_out = dict_ref (dict);
+
+        __wake (args);
+
+        return 0;
+
+}
+
+int
+syncop_xattrop (xlator_t *subvol, loc_t *loc, gf_xattrop_flags_t flags,
+                dict_t *dict, dict_t *xdata_in, dict_t **dict_out,
+                dict_t **xdata_out)
+{
+        struct syncargs args = {0, };
+
+        SYNCOP (subvol, (&args), syncop_xattrop_cbk, subvol->fops->xattrop,
+                loc, flags, dict, xdata_in);
+
+        if (xdata_out)
+                *xdata_out = args.xdata;
+        else if (args.xdata)
+                dict_unref (args.xdata);
+
+        if (dict_out)
+                *dict_out = args.dict_out;
+        else if (args.dict_out)
+                dict_unref (args.dict_out);
+
+        if (args.op_ret < 0)
+                return -args.op_errno;
+
+        return args.op_ret;
+}
+
 int
 syncop_fxattrop (xlator_t *subvol, fd_t *fd, gf_xattrop_flags_t flags,
-                 dict_t *dict, dict_t *xdata_in, dict_t **xdata_out)
+                 dict_t *dict, dict_t *xdata_in, dict_t **dict_out,
+                 dict_t **xdata_out)
 {
         struct syncargs args = {0, };
 
@@ -3121,6 +3189,11 @@ syncop_fxattrop (xlator_t *subvol, fd_t *fd, gf_xattrop_flags_t flags,
         else if (args.xdata)
                 dict_unref (args.xdata);
 
+        if (dict_out)
+                *dict_out = args.dict_out;
+        else if (args.dict_out)
+                dict_unref (args.dict_out);
+
         if (args.op_ret < 0)
                 return -args.op_errno;
 
