@@ -15,9 +15,11 @@
 #include "defaults.h"
 #include "libglusterfs-messages.h"
 
-#define SET_DEFAULT_FOP(fn) do {			\
-                if (!xl->fops->fn)			\
-                        xl->fops->fn = default_##fn;	\
+#define SET_DEFAULT_FOP(fn) do {                                        \
+                if (!xl->fops->fn)                                      \
+                        xl->fops->fn = default_##fn;                    \
+                if (!xl->pass_through_fops->fn)                         \
+                        xl->pass_through_fops->fn = default_##fn;       \
         } while (0)
 
 #define SET_DEFAULT_CBK(fn) do {			\
@@ -54,6 +56,9 @@ fill_defaults (xlator_t *xl)
                 return;
         }
 
+        if (!xl->pass_through_fops)
+                xl->pass_through_fops = default_fops;
+
         SET_DEFAULT_FOP (create);
         SET_DEFAULT_FOP (open);
         SET_DEFAULT_FOP (stat);
@@ -217,6 +222,7 @@ xlator_volopt_dynload (char *xlator_type, void **dl_handle,
 
 int xlator_dynload_oldway (xlator_t *xl)
 {
+        int                i = 0;
         int                ret = -1;
         void              *handle = NULL;
         volume_opt_list_t *vol_opt = NULL;
@@ -300,6 +306,12 @@ int xlator_dynload_oldway (xlator_t *xl)
         INIT_LIST_HEAD (&vol_opt->list);
         list_add_tail (&vol_opt->list, &xl->volume_options);
 
+        /* make sure 'min' is set to high value, so it would be
+           properly set later */
+        for (i = 0; i < GF_FOP_MAXVALUE; i++) {
+                xl->stats.interval.latencies[i].min = 0xffffffff;
+        }
+
         ret = 0;
 
 out:
@@ -379,6 +391,13 @@ int xlator_dynload_newway (xlator_t *xl)
                               xl->name);
         }
 
+        xl->pass_through_fops = xlapi->pass_through_fops;
+        if (!xl->pass_through_fops) {
+                gf_msg_trace ("xlator", 0, "%s: method missing (pass_through_fops), "
+                              "falling back to default",
+                              xl->name);
+        }
+
         vol_opt = GF_CALLOC (1, sizeof (volume_opt_list_t),
                              gf_common_mt_volume_opt_list_t);
         if (!vol_opt) {
