@@ -10,16 +10,14 @@
 
 #include "event.h"
 #include "rpcsvc.h"
-#include "server.h"
 
 void
-rpcsvc_autoscale_threads (glusterfs_ctx_t *ctx, int incr, xlator_t *this)
+rpcsvc_autoscale_threads (glusterfs_ctx_t *ctx, rpcsvc_t *rpc, int incr)
 {
         struct event_pool       *pool           = ctx->event_pool;
-        server_conf_t           *conf           = this->private;
         int                      thread_count   = pool->eventthreadcount;
 
         pool->auto_thread_count += incr;
         (void) event_reconfigure_threads (pool, thread_count+incr);
-        rpcsvc_ownthread_reconf (conf->rpc, pool->eventthreadcount);
+        rpcsvc_ownthread_reconf (rpc, pool->eventthreadcount);
 }
