@@ -104,7 +104,6 @@ call_bail (void *data)
         struct saved_frame    *tmp = NULL;
         char                   frame_sent[256] = {0,};
         struct timespec        timeout = {0,};
-        struct iovec           iov = {0,};
         char                   peerid[UNIX_PATH_MAX] = {0};
         gf_boolean_t           need_unref = _gf_false;
 
@@ -186,7 +185,7 @@ call_bail (void *data)
 
                 clnt = rpc_clnt_ref (clnt);
                 trav->rpcreq->rpc_status = -1;
-		trav->rpcreq->cbkfn (trav->rpcreq, &iov, 1, trav->frame);
+		trav->rpcreq->cbkfn (trav->rpcreq, NULL, 0, trav->frame);
 
                 rpc_clnt_reply_deinit (trav->rpcreq, clnt->reqpool);
                 clnt = rpc_clnt_unref (clnt);
@@ -324,7 +323,6 @@ saved_frames_unwind (struct saved_frames *saved_frames)
 	struct saved_frame   *trav = NULL;
 	struct saved_frame   *tmp = NULL;
         char                  timestr[1024] = {0,};
-        struct iovec          iov = {0,};
 
         list_splice_init (&saved_frames->lk_sf.list, &saved_frames->sf.list);
 
@@ -351,7 +349,7 @@ saved_frames_unwind (struct saved_frames *saved_frames)
 		saved_frames->count--;
 
                 trav->rpcreq->rpc_status = -1;
-                trav->rpcreq->cbkfn (trav->rpcreq, &iov, 1, trav->frame);
+                trav->rpcreq->cbkfn (trav->rpcreq, NULL, 0, trav->frame);
 
                 rpc_clnt_reply_deinit (trav->rpcreq,
                                        trav->rpcreq->conn->rpc_clnt->reqpool);
@@ -937,10 +935,20 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
         case RPC_TRANSPORT_DISCONNECT:
         {
                 rpc_clnt_handle_disconnect (clnt, conn);
-                /* reset auth_type to use v2 (if its not auth-null), it
-                   would be set to appropriate type in handshake again */
-                if (clnt->auth_value)
-                        clnt->auth_value = AUTH_GLUSTERFS_v2;
+                /* The auth_value was being reset to AUTH_GLUSTERFS_v2.
+                 *    if (clnt->auth_value)
+                 *           clnt->auth_value = AUTH_GLUSTERFS_v2;
+                 * It should not be reset here. The disconnect during
+                 * portmap request can race with handshake. If handshake
+                 * happens first and disconnect later, auth_value would set
+                 * to default value and it never sets back to actual auth_value
+                 * supported by server. But it's important to set to lower
+                 * version supported in the case where the server downgrades.
+                 * So moving this code to RPC_TRANSPORT_CONNECT. Note that
+                 * CONNECT cannot race with handshake as by nature it is
+                 * serialized with handhake. An handshake can happen only
+                 * on a connected transport and hence its strictly serialized.
+                 */
                 break;
         }
 
@@ -1001,6 +1009,12 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
                  * for just one successful attempt */
                 conn->config.remote_port = 0;
 
+                /* auth value should be set to lower version available
+                 * and will be set to appropriate version supported by
+                 * server after the handshake.
+                 */
+                if (clnt->auth_value)
+                        clnt->auth_value = AUTH_GLUSTERFS_v2;
                 if (clnt->notifyfn)
                         ret = clnt->notifyfn (clnt, clnt->mydata,
                                               RPC_CLNT_CONNECT, NULL);
@@ -1954,10 +1968,20 @@ rpc_clnt_disable (struct rpc_clnt *rpc)
 
         if (trans) {
                 rpc_transport_disconnect (trans, _gf_true);
-                /* reset auth_type to use v2 (if its not auth-null), it
-                   would be set to appropriate type in handshake again */
-                if (rpc->auth_value)
-                        rpc->auth_value = AUTH_GLUSTERFS_v2;
+                /* The auth_value was being reset to AUTH_GLUSTERFS_v2.
+                 *    if (clnt->auth_value)
+                 *           clnt->auth_value = AUTH_GLUSTERFS_v2;
+                 * It should not be reset here. The disconnect during
+                 * portmap request can race with handshake. If handshake
+                 * happens first and disconnect later, auth_value would set
+                 * to default value and it never sets back to actual auth_value
+                 * supported by server. But it's important to set to lower
+                 * version supported in the case where the server downgrades.
+                 * So moving this code to RPC_TRANSPORT_CONNECT. Note that
+                 * CONNECT cannot race with handshake as by nature it is
+                 * serialized with handhake. An handshake can happen only
+                 * on a connected transport and hence its strictly serialized.
+                 */
         }
 
         if (unref)
@@ -2017,10 +2041,20 @@ rpc_clnt_disconnect (struct rpc_clnt *rpc)
 
         if (trans) {
                 rpc_transport_disconnect (trans, _gf_true);
-                /* reset auth_type to use v2 (if its not auth-null), it
-                   would be set to appropriate type in handshake again */
-                if (rpc->auth_value)
-                        rpc->auth_value = AUTH_GLUSTERFS_v2;
+                /* The auth_value was being reset to AUTH_GLUSTERFS_v2.
+                 *    if (clnt->auth_value)
+                 *           clnt->auth_value = AUTH_GLUSTERFS_v2;
+                 * It should not be reset here. The disconnect during
+                 * portmap request can race with handshake. If handshake
+                 * happens first and disconnect later, auth_value would set
+                 * to default value and it never sets back to actual auth_value
+                 * supported by server. But it's important to set to lower
+                 * version supported in the case where the server downgrades.
+                 * So moving this code to RPC_TRANSPORT_CONNECT. Note that
+                 * CONNECT cannot race with handshake as by nature it is
+                 * serialized with handhake. An handshake can happen only
+                 * on a connected transport and hence its strictly serialized.
+                 */
         }
         if (unref)
                 rpc_clnt_unref (rpc);
