@@ -180,11 +180,11 @@ struct rpc_transport {
                                               */
 
         void                      *private;
-        struct _client_t          *xl_private;
+        struct _client            *xl_private;
         void                      *xl;       /* Used for THIS */
         void                      *mydata;
         pthread_mutex_t            lock;
-        int32_t                    refcount;
+        gf_atomic_t                refcount;
 
         int32_t                    outstanding_rpc_count;
 
