@@ -11,11 +11,6 @@
 #ifndef _XDR_RPC_H_
 #define _XDR_RPC_H_
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #ifndef GF_SOLARIS_HOST_OS
 #include <rpc/rpc.h>
 #endif
@@ -36,6 +31,7 @@ typedef enum {
         AUTH_GLUSTERFS = 5,
         AUTH_GLUSTERFS_v2 = 390039, /* using a number from  'unused' range,
                                        from the list available in RFC5531 */
+        AUTH_GLUSTERFS_v3 = 390040, /* this too is unused */
 } gf_rpc_authtype_t;
 
 /* Converts a given network buffer from its XDR format to a structure
@@ -79,7 +75,7 @@ xdr_to_auth_unix_cred (char *msgbuf, int msglen, struct authunix_parms *au,
 #define rpc_call_verf_len(call)         (rpc_opaque_auth_len ((&(call)->ru.RM_cmb.cb_verf)))
 
 
-#ifdef GF_DARWIN_HOST_OS
+#if defined(GF_DARWIN_HOST_OS) || defined (IPV6_DEFAULT)
 #define GF_PRI_RPC_XID          PRIu32
 #define GF_PRI_RPC_VERSION      PRIu32
 #define GF_PRI_RPC_PROG_ID      PRIu32
