@@ -1,10 +1,11 @@
-#!/usr/bin/env python
+#!/usr/bin/python2
 #
 # Create a unix domain socket and test if it is a socket (and not a fifo/pipe).
 #
 # Author: Niels de Vos <ndevos@redhat.com>
 #
 
+from __future__ import print_function
 import os
 import stat
 import sys
@@ -13,7 +14,7 @@ import socket
 ret = 1
 
 if len(sys.argv) != 2:
-        print 'Usage: %s <socket>' % (sys.argv[0])
+        print('Usage: %s <socket>' % (sys.argv[0]))
         sys.exit(ret)
 
 path = sys.argv[1]
