@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python2
 # -*- coding: utf-8 -*-
 
 # Copyright (c) 2015 Red Hat, Inc. <http://www.redhat.com/>
@@ -42,7 +42,8 @@ def brickfind_crawl(brick, args):
             path = path.strip()
             path = path[brick_path_len+1:]
             output_write(fout, path, args.output_prefix,
-                         encode=(not args.no_encode), tag=args.tag)
+                         encode=(not args.no_encode), tag=args.tag,
+                         field_separator=args.field_separator)
 
         ignore_dirs = [os.path.join(brick, dirname)
                        for dirname in
@@ -61,9 +62,9 @@ def _get_args():
 
     parser.add_argument("session", help="Session Name")
     parser.add_argument("volume", help="Volume Name")
+    parser.add_argument("node", help="Node Name")
     parser.add_argument("brick", help="Brick Name")
     parser.add_argument("outfile", help="Output File")
-    parser.add_argument("start", help="Start Time", type=float)
     parser.add_argument("tag", help="Tag to prefix file name with")
     parser.add_argument("--only-query", help="Only query, No session update",
                         action="store_true")
@@ -73,6 +74,8 @@ def _get_args():
                         action="store_true")
     parser.add_argument("--output-prefix", help="File prefix in output",
                         default=".")
+    parser.add_argument("--field-separator", help="Field separator",
+                        default=" ")
 
     return parser.parse_args()
 
