@@ -45,8 +45,10 @@ enum gf_afr_mem_types_ {
 	gf_afr_mt_subvol_healer_t,
 	gf_afr_mt_spbc_timeout_t,
         gf_afr_mt_spb_status_t,
-        gf_afr_mt_replace_brick_t,
-        gf_afr_mt_end
+        gf_afr_mt_empty_brick_t,
+        gf_afr_mt_child_latency_t,
+        gf_afr_mt_atomic_t,
+    gf_afr_mt_end
 };
 #endif
 
