@@ -101,7 +101,7 @@ afr_read_txn_refresh_done (call_frame_t *frame, xlator_t *this, int err)
 	read_subvol = afr_read_subvol_select_by_policy (inode, this,
 							local->readable, NULL);
 	if (read_subvol == -1) {
-                err = -EIO;
+                err = EIO;
                 goto readfn;
         }
 
@@ -120,7 +120,7 @@ readfn:
         }
 
         if (read_subvol == -1) {
-                AFR_SET_ERROR_AND_CHECK_SPLIT_BRAIN (-1, -err);
+                AFR_SET_ERROR_AND_CHECK_SPLIT_BRAIN (-1, err);
         }
         afr_read_txn_wind (frame, this, read_subvol);
 
@@ -228,7 +228,7 @@ afr_read_txn (call_frame_t *frame, xlator_t *this, inode_t *inode,
 
         if (priv->quorum_count && !afr_has_quorum (local->child_up, this)) {
                 local->op_ret = -1;
-                local->op_errno = ENOTCONN;
+                local->op_errno = afr_quorum_errno(priv);
                 read_subvol = -1;
                 goto read;
         }
