@@ -187,7 +187,7 @@ __afr_selfheal_heal_dirent (call_frame_t *frame, xlator_t *this, fd_t *fd,
 
         if (replies[source].op_ret == 0) {
                 ret = afr_lookup_and_heal_gfid (this, fd->inode, name,
-                                                inode, replies, source,
+                                                inode, replies, source, sources,
                                              &replies[source].poststat.ia_gfid);
                 if (ret)
                         return ret;
@@ -320,7 +320,7 @@ __afr_selfheal_merge_dirent (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	}
 
         ret = afr_lookup_and_heal_gfid (this, fd->inode, name, inode, replies,
-                                        source,
+                                        source, sources,
                                         &replies[source].poststat.ia_gfid);
         if (ret)
                 return ret;
