@@ -495,11 +495,10 @@ afr_txn_arbitrate_fop (call_frame_t *frame, xlator_t *this)
                 local->op_errno =  ENOTCONN;
                 for (i = 0; i < priv->child_count; i++)
                         local->transaction.failed_subvols[i] = 1;
-                afr_changelog_post_op (frame, this);/*uninherit should happen*/
-        } else {
-                afr_transaction_fop (frame, this);
         }
 
+        afr_transaction_fop (frame, this);
+
         return;
 }
 
@@ -529,13 +528,6 @@ afr_transaction_perform_fop (call_frame_t *frame, xlator_t *this)
                                 local->transaction.failed_subvols[i] = 1;
                 }
         }
-        /*  Perform fops with the lk-owner from top xlator.
-         *  Eg: lk-owner of posix-lk and flush should be same,
-         *  flush cant clear the  posix-lks without that lk-owner.
-         */
-        afr_save_lk_owner (frame);
-        frame->root->lk_owner =
-                local->transaction.main_frame->root->lk_owner;
 
 	if (local->pre_op_compat)
 		/* old mode, pre-op was done as afr_changelog_do()
@@ -561,6 +553,14 @@ afr_transaction_perform_fop (call_frame_t *frame, xlator_t *this)
         }
 
 fop:
+        /*  Perform fops with the lk-owner from top xlator.
+         *  Eg: lk-owner of posix-lk and flush should be same,
+         *  flush cant clear the  posix-lks without that lk-owner.
+         */
+        afr_save_lk_owner (frame);
+        frame->root->lk_owner =
+                local->transaction.main_frame->root->lk_owner;
+
         if (priv->arbiter_count == 1) {
                 afr_txn_arbitrate_fop (frame, this);
         } else {
@@ -2327,6 +2327,7 @@ __afr_eager_lock_handle (afr_local_t *local, gf_boolean_t *take_lock,
 
         if (local->fd && !afr_are_multiple_fds_opened (local, this)) {
                 local->transaction.eager_lock_on = _gf_true;
+                afr_set_lk_owner (local->transaction.frame, this, local->inode);
         }
 
         lock = &local->inode_ctx->lock[local->transaction.type];
@@ -2430,8 +2431,6 @@ lock_phase:
         if (!local->transaction.eager_lock_on) {
                 afr_set_lk_owner (local->transaction.frame, this,
                                   local->transaction.frame->root);
-        } else {
-                afr_set_lk_owner (local->transaction.frame, this, local->inode);
         }
 
 
@@ -2452,7 +2451,7 @@ afr_write_txn_refresh_done (call_frame_t *frame, xlator_t *this, int err)
         afr_local_t   *local           = frame->local;
 
         if (err) {
-                AFR_SET_ERROR_AND_CHECK_SPLIT_BRAIN(-1, -err);
+                AFR_SET_ERROR_AND_CHECK_SPLIT_BRAIN(-1, err);
                 goto fail;
         }
 
@@ -2478,6 +2477,11 @@ afr_transaction (call_frame_t *frame, xlator_t *this, afr_transaction_type type)
 
         local->transaction.type   = type;
 
+        if (priv->quorum_count && !afr_has_quorum (local->child_up, this)) {
+                ret = -afr_quorum_errno(priv);
+                goto out;
+        }
+
         if (!afr_is_consistent_io_possible (local, priv, &ret)) {
                 ret = -ret; /*op_errno to ret conversion*/
                 goto out;
