@@ -17,21 +17,12 @@ void
 afr_transaction_fop_failed (call_frame_t *frame, xlator_t *this,
 			    int child_index);
 
-int
-afr_lock_server_count (afr_private_t *priv, afr_transaction_type type);
-
-afr_inodelk_t*
-afr_get_inodelk (afr_internal_lock_t *int_lock, char *dom);
-
 int32_t
 afr_transaction (call_frame_t *frame, xlator_t *this, afr_transaction_type type);
 
 int
 afr_set_pending_dict (afr_private_t *priv, dict_t *xattr, int32_t **pending);
 
-void
-afr_set_delayed_post_op (call_frame_t *frame, xlator_t *this);
-
 void
 afr_delayed_changelog_wake_up (xlator_t *this, fd_t *fd);
 
@@ -46,8 +37,12 @@ int afr_read_txn (call_frame_t *frame, xlator_t *this, inode_t *inode,
 
 int afr_read_txn_continue (call_frame_t *frame, xlator_t *this, int subvol);
 
-int __afr_txn_write_fop (call_frame_t *frame, xlator_t *this);
-int __afr_txn_write_done (call_frame_t *frame, xlator_t *this);
+void
+afr_pending_read_increment (afr_private_t *priv, int child_index);
+
+void
+afr_pending_read_decrement (afr_private_t *priv, int child_index);
+
 call_frame_t *afr_transaction_detach_fop_frame (call_frame_t *frame);
 gf_boolean_t afr_has_quorum (unsigned char *subvols, xlator_t *this);
 gf_boolean_t afr_needs_changelog_update (afr_local_t *local);
@@ -58,7 +53,14 @@ afr_pick_error_xdata (afr_local_t *local, afr_private_t *priv,
                       inode_t *inode1, unsigned char *readable1,
                       inode_t *inode2, unsigned char *readable2);
 int
-afr_pre_op_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                       int op_ret, int op_errno,
-                       void *data, dict_t *xdata);
+afr_transaction_resume (call_frame_t *frame, xlator_t *this);
+
+int
+afr_lock (call_frame_t *frame, xlator_t *this);
+
+void
+afr_delayed_changelog_wake_up_cbk (void *data);
+
+int
+afr_fill_ta_loc (xlator_t *this, loc_t *loc);
 #endif /* __TRANSACTION_H__ */
