@@ -490,37 +490,66 @@ err:
         return 0;
 }
 
+
+/*
+ * If the hashed subvolumes of both source and dst are the different,
+ * lock in dictionary order of hashed subvol->name. This is important
+ * in case the parent directory is the same for both src and dst to
+ * prevent inodelk deadlocks when racing with a fix-layout op on the parent.
+ *
+ * If the hashed subvols are the same, use the gfid/name to determine
+ * the order of taking locks to prevent entrylk deadlocks when the parent
+ * dirs are the same.
+ *
+ */
 static void
 dht_order_rename_lock (call_frame_t *frame, loc_t **loc, xlator_t **subvol)
 {
-        dht_local_t        *local                       = NULL;
-        char                src[GF_UUID_BNAME_BUF_SIZE] = {0};
-        char                dst[GF_UUID_BNAME_BUF_SIZE] = {0};
+        int ret                 = 0;
+        dht_local_t   *local    = NULL;
+        char           src[GF_UUID_BNAME_BUF_SIZE] = {0};
+        char           dst[GF_UUID_BNAME_BUF_SIZE] = {0};
+
 
         local = frame->local;
 
-        if (local->loc.pargfid)
-                uuid_utoa_r (local->loc.pargfid, src);
-        else if (local->loc.parent)
-                uuid_utoa_r (local->loc.parent->gfid, src);
-
-        strcat (src, local->loc.name);
-
-        if (local->loc2.pargfid)
-                uuid_utoa_r (local->loc2.pargfid, dst);
-        else if (local->loc2.parent)
-                uuid_utoa_r (local->loc2.parent->gfid, dst);
-
-        strcat (dst, local->loc2.name);
-
-        if (strcmp(src, dst) > 0) {
-                local->current = &local->lock[1];
-                *loc = &local->loc2;
-                *subvol = local->dst_hashed;
+        if (local->src_hashed->name == local->dst_hashed->name) {
+                ret = 0;
         } else {
+                ret = strcmp (local->src_hashed->name, local->dst_hashed->name);
+        }
+
+        if (ret == 0) {
+
+                /* hashed subvols are the same for src and dst */
+                /* Entrylks need to be ordered*/
+                if (local->loc.pargfid)
+                        uuid_utoa_r (local->loc.pargfid, src);
+                else if (local->loc.parent)
+                        uuid_utoa_r (local->loc.parent->gfid, src);
+
+                strcat (src, local->loc.name);
+
+                if (local->loc2.pargfid)
+                        uuid_utoa_r (local->loc2.pargfid, dst);
+                else if (local->loc2.parent)
+                        uuid_utoa_r (local->loc2.parent->gfid, dst);
+
+                strcat (dst, local->loc2.name);
+                ret = strcmp (src, dst);
+        }
+
+        if (ret <= 0) {
+                /*inodelk in dictionary order of hashed subvol names*/
+                /*entrylk in dictionary order of gfid/basename */
                 local->current = &local->lock[0];
                 *loc = &local->loc;
                 *subvol = local->src_hashed;
+
+        } else {
+                local->current = &local->lock[1];
+                *loc = &local->loc2;
+                *subvol = local->dst_hashed;
         }
 
         return;
@@ -1518,6 +1547,8 @@ dht_rename_lock_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         dict_t      *xattr_req                  = NULL;
         dht_conf_t  *conf                       = NULL;
         int          i                          = 0;
+        int          count                      = 0;
+
 
         local = frame->local;
         conf = this->private;
@@ -1557,7 +1588,7 @@ dht_rename_lock_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 goto done;
         }
 
-        local->call_cnt = local->lock[0].layout.parent_layout.lk_count;
+        count = local->call_cnt = local->lock[0].layout.parent_layout.lk_count;
 
         /* Why not use local->lock.locks[?].loc for lookup post lock phase
          * ---------------------------------------------------------------
@@ -1577,7 +1608,7 @@ dht_rename_lock_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
          * exists with the name that the client requested with.
          * */
 
-        for (i = 0; i < local->lock[0].layout.parent_layout.lk_count; i++) {
+        for (i = 0; i < count; i++) {
                 STACK_WIND_COOKIE (frame, dht_rename_lookup_cbk, (void *)(long)i,
                                    local->lock[0].layout.parent_layout.locks[i]->xl,
                                    local->lock[0].layout.parent_layout.locks[i]->xl->fops->lookup,
@@ -1618,14 +1649,16 @@ dht_rename_lock (call_frame_t *frame)
                 goto err;
 
         lk_array[0] = dht_lock_new (frame->this, local->src_cached, &local->loc,
-                                    F_WRLCK, DHT_FILE_MIGRATE_DOMAIN, NULL);
+                                    F_WRLCK, DHT_FILE_MIGRATE_DOMAIN, NULL,
+                                    FAIL_ON_ANY_ERROR);
         if (lk_array[0] == NULL)
                 goto err;
 
         if (local->dst_cached) {
                 lk_array[1] = dht_lock_new (frame->this, local->dst_cached,
                                             &local->loc2, F_WRLCK,
-                                            DHT_FILE_MIGRATE_DOMAIN, NULL);
+                                            DHT_FILE_MIGRATE_DOMAIN, NULL,
+                                            FAIL_ON_ANY_ERROR);
                 if (lk_array[1] == NULL)
                         goto err;
         }
@@ -1634,7 +1667,7 @@ dht_rename_lock (call_frame_t *frame)
         local->lock[0].layout.parent_layout.lk_count = count;
 
         ret = dht_blocking_inodelk (frame, lk_array, count,
-                                    FAIL_ON_ANY_ERROR, dht_rename_lock_cbk);
+                                    dht_rename_lock_cbk);
         if (ret < 0) {
                 local->lock[0].layout.parent_layout.locks = NULL;
                 local->lock[0].layout.parent_layout.lk_count = 0;
@@ -1659,13 +1692,14 @@ int
 dht_rename (call_frame_t *frame, xlator_t *this,
             loc_t *oldloc, loc_t *newloc, dict_t *xdata)
 {
-        xlator_t    *src_cached = NULL;
-        xlator_t    *src_hashed = NULL;
-        xlator_t    *dst_cached = NULL;
-        xlator_t    *dst_hashed = NULL;
-        int          op_errno = -1;
-        int          ret = -1;
-        dht_local_t *local = NULL;
+        xlator_t    *src_cached             = NULL;
+        xlator_t    *src_hashed             = NULL;
+        xlator_t    *dst_cached             = NULL;
+        xlator_t    *dst_hashed             = NULL;
+        int          op_errno               = -1;
+        int          ret                    = -1;
+        dht_local_t *local                  = NULL;
+        dht_conf_t  *conf                   = NULL;
         char         gfid[GF_UUID_BUF_SIZE] = {0};
 
         VALIDATE_OR_GOTO (frame, err);
@@ -1673,6 +1707,13 @@ dht_rename (call_frame_t *frame, xlator_t *this,
         VALIDATE_OR_GOTO (oldloc, err);
         VALIDATE_OR_GOTO (newloc, err);
 
+        conf = this->private;
+
+        if (conf->subvolume_cnt == 1) {
+                default_rename (frame, this, oldloc, newloc, xdata);
+                return 0;
+        }
+
         gf_uuid_unparse(oldloc->inode->gfid, gfid);
 
         src_hashed = dht_subvol_get_hashed (this, oldloc);
