@@ -164,7 +164,9 @@ gf_changelog_parse_binary (xlator_t *this,
         char    current_mover    = ' ';
         size_t  blen             = 0;
         int     parse_err        = 0;
-        char ascii[LINE_BUFSIZE] = {0,};
+        char   *ascii            = NULL;
+
+        ascii = GF_CALLOC (LINE_BUFSIZE, sizeof(char), gf_common_mt_char);
 
         nleft = stbuf->st_size;
 
@@ -245,6 +247,8 @@ gf_changelog_parse_binary (xlator_t *this,
                         CHANGELOG_LIB_MSG_MUNMAP_FAILED,
                         "munmap() error");
  out:
+        if (ascii)
+                GF_FREE (ascii);
         return ret;
 }
 
@@ -272,9 +276,11 @@ gf_changelog_parse_ascii (xlator_t *this,
         char         *mover         = NULL;
         int           parse_err     = 0;
         char          current_mover = ' ';
-        char ascii[LINE_BUFSIZE]    = {0,};
+        char         *ascii         = NULL;
         const char   *fopname       = NULL;
 
+        ascii = GF_CALLOC (LINE_BUFSIZE, sizeof(char), gf_common_mt_char);
+
         nleft = stbuf->st_size;
 
         start = mmap (NULL, nleft, PROT_READ, MAP_PRIVATE, from_fd, 0);
@@ -432,34 +438,12 @@ gf_changelog_parse_ascii (xlator_t *this,
                         "munmap() error");
 
  out:
+        if (ascii)
+                GF_FREE (ascii);
+
         return ret;
 }
 
-#define COPY_BUFSIZE  8192
-static int
-gf_changelog_copy (xlator_t *this, int from_fd, int to_fd)
-{
-        ssize_t size                  = 0;
-        char   buffer[COPY_BUFSIZE+1] = {0,};
-
-        while (1) {
-                size = sys_read (from_fd, buffer, COPY_BUFSIZE);
-                if (size <= 0)
-                        break;
-
-                if (gf_changelog_write (to_fd,
-                                        buffer, size) != size) {
-                        gf_msg (this->name, GF_LOG_ERROR, 0,
-                                CHANGELOG_LIB_MSG_COPY_FROM_BUFFER_FAILED,
-                                "error processing ascii changlog");
-                        size = -1;
-                        break;
-                }
-        }
-
-        return (size < 0 ? -1 : 0);
-}
-
 static int
 gf_changelog_decode (xlator_t *this, gf_changelog_journal_t *jnl,
                      int from_fd, int to_fd, struct stat *stbuf, int *zerob)
@@ -503,8 +487,9 @@ gf_changelog_decode (xlator_t *this, gf_changelog_journal_t *jnl,
         /**
          * start processing after the header
          */
-        sys_lseek (from_fd, elen, SEEK_SET);
-
+        if (sys_lseek (from_fd, elen, SEEK_SET) < 0) {
+                goto out;
+        }
         switch (encoding) {
         case CHANGELOG_ENCODE_BINARY:
                 /**
@@ -521,11 +506,9 @@ gf_changelog_decode (xlator_t *this, gf_changelog_journal_t *jnl,
                                                 to_fd, elen, stbuf,
                                                 version_idx);
                 break;
-        default:
-                ret = gf_changelog_copy (this, from_fd, to_fd);
         }
 
- out:
+out:
         return ret;
 }
 
@@ -554,10 +537,11 @@ gf_changelog_publish (xlator_t *this,
 
         ret = sys_rename (to_path, dest);
         if (ret) {
-                gf_msg (this->name, GF_LOG_ERROR, errno,
-                        CHANGELOG_LIB_MSG_RENAME_FAILED,
-                        "error moving %s to processing dir",
-                        to_path);
+                gf_smsg (this->name, GF_LOG_ERROR, errno,
+                         CHANGELOG_LIB_MSG_RENAME_FAILED,
+                         "error moving changelog to processing dir",
+                         "path=%s", to_path,
+                         NULL);
         }
 
 out:
@@ -580,18 +564,21 @@ gf_changelog_consume (xlator_t *this,
         ret = sys_stat (from_path, &stbuf);
         if (ret || !S_ISREG(stbuf.st_mode)) {
                 ret = -1;
-                gf_msg (this->name, GF_LOG_ERROR, errno,
-                        CHANGELOG_LIB_MSG_STAT_FAILED,
-                        "stat failed on changelog file: %s", from_path);
+                gf_smsg (this->name, GF_LOG_ERROR, errno,
+                         CHANGELOG_LIB_MSG_STAT_FAILED,
+                         "stat failed on changelog file",
+                         "path=%s", from_path,
+                         NULL);
                 goto out;
         }
 
         fd1 = open (from_path, O_RDONLY);
         if (fd1 < 0) {
-                gf_msg (this->name, GF_LOG_ERROR, errno,
-                        CHANGELOG_LIB_MSG_OPEN_FAILED,
-                        "cannot open changelog file: %s",
-                        from_path);
+                gf_smsg (this->name, GF_LOG_ERROR, errno,
+                         CHANGELOG_LIB_MSG_OPEN_FAILED,
+                         "cannot open changelog file",
+                         "path=%s", from_path,
+                         NULL);
                 goto out;
         }
 
@@ -603,10 +590,11 @@ gf_changelog_consume (xlator_t *this,
         fd2 = open (to_path, O_CREAT | O_TRUNC | O_RDWR,
                     S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
         if (fd2 < 0) {
-                gf_msg (this->name, GF_LOG_ERROR, errno,
-                        CHANGELOG_LIB_MSG_OPEN_FAILED,
-                        "cannot create ascii changelog file %s",
-                        to_path);
+                gf_smsg (this->name, GF_LOG_ERROR, errno,
+                         CHANGELOG_LIB_MSG_OPEN_FAILED,
+                         "cannot create ascii changelog file",
+                         "path=%s", to_path,
+                         NULL);
                 goto close_fd;
         } else {
                 ret = gf_changelog_decode (this, jnl, fd1,
@@ -621,10 +609,11 @@ gf_changelog_consume (xlator_t *this,
                                 goto close_fd;
                         ret = sys_rename (to_path, dest);
                         if (ret)
-                                gf_msg (this->name, GF_LOG_ERROR, errno,
-                                        CHANGELOG_LIB_MSG_RENAME_FAILED,
-                                        "error moving %s to processing dir",
-                                        to_path);
+                                gf_smsg (this->name, GF_LOG_ERROR, errno,
+                                         CHANGELOG_LIB_MSG_RENAME_FAILED,
+                                         "error moving changelog to processing dir",
+                                         "path=%s", to_path,
+                                         NULL);
                 }
 
                 /* remove it from .current if it's an empty file */
@@ -632,10 +621,11 @@ gf_changelog_consume (xlator_t *this,
                         /* zerob changelogs must be unlinked */
                         ret = sys_unlink (to_path);
                         if (ret)
-                                gf_msg (this->name, GF_LOG_ERROR, errno,
-                                        CHANGELOG_LIB_MSG_UNLINK_FAILED,
-                                        "could not unlink %s",
-                                        to_path);
+                                gf_smsg (this->name, GF_LOG_ERROR, errno,
+                                         CHANGELOG_LIB_MSG_UNLINK_FAILED,
+                                         "could not unlink empty changelog",
+                                         "path=%s", to_path,
+                                         NULL);
                 }
         }
 
@@ -670,7 +660,9 @@ gf_changelog_process (void *data)
 
                         entry = list_first_entry (&jnl_proc->entries,
                                                   gf_changelog_entry_t, list);
-                        list_del (&entry->list);
+                        if (entry)
+                                list_del (&entry->list);
+
                         jnl_proc->waiting = _gf_false;
                 }
                 pthread_mutex_unlock (&jnl_proc->lock);
@@ -700,6 +692,7 @@ gf_changelog_queue_journal (gf_changelog_processor_t *jnl_proc,
 
         len = strlen (event->u.journal.path);
         (void)memcpy (entry->path, event->u.journal.path, len+1);
+        entry->path[len] = '\0';
 
         pthread_mutex_lock (&jnl_proc->lock);
         {
@@ -854,10 +847,11 @@ gf_changelog_open_dirs (xlator_t *this, gf_changelog_journal_t *jnl)
                          jnl->jnl_working_dir);
         ret = recursive_rmdir (jnl->jnl_current_dir);
         if (ret) {
-                gf_msg (this->name, GF_LOG_ERROR, errno,
-                        CHANGELOG_LIB_MSG_FAILED_TO_RMDIR,
-                        "Failed to rmdir: %s",
-                        jnl->jnl_current_dir);
+                gf_smsg (this->name, GF_LOG_ERROR, errno,
+                         CHANGELOG_LIB_MSG_FAILED_TO_RMDIR,
+                         "Failed to rmdir",
+                         "path=%s", jnl->jnl_current_dir,
+                         NULL);
                 goto out;
         }
         ret = mkdir_p (jnl->jnl_current_dir, 0600, _gf_false);
@@ -878,10 +872,11 @@ gf_changelog_open_dirs (xlator_t *this, gf_changelog_journal_t *jnl)
                          jnl->jnl_working_dir);
         ret = recursive_rmdir (jnl->jnl_processing_dir);
         if (ret) {
-                gf_msg (this->name, GF_LOG_ERROR, errno,
-                        CHANGELOG_LIB_MSG_FAILED_TO_RMDIR,
-                        "Failed to rmdir: %s",
-                        jnl->jnl_processing_dir);
+                gf_smsg (this->name, GF_LOG_ERROR, errno,
+                         CHANGELOG_LIB_MSG_FAILED_TO_RMDIR,
+                         "Failed to rmdir",
+                         "path=%s", jnl->jnl_processing_dir,
+                         NULL);
                 goto out;
         }
 
