@@ -46,9 +46,10 @@ init (xlator_t *this)
         if (!priv)
                 goto out;
 
+        this->private = priv;
+
         GF_OPTION_INIT ("read-only", priv->readonly_or_worm_enabled, bool, out);
 
-        this->private = priv;
         ret = 0;
 out:
         return ret;
@@ -125,6 +126,9 @@ struct volume_options options[] = {
 	{ .key  = {"read-only"},
           .type = GF_OPTION_TYPE_BOOL,
           .default_value = "off",
+          /*.validate_fn = validate_boolean,*/
+          .op_version = {1},
+          .flags = OPT_FLAG_SETTABLE,
           .description = "When \"on\", makes a volume read-only. It is turned "
                          "\"off\" by default."
         },
