@@ -41,7 +41,8 @@ GLFS_MSGID(SHARD,
         SHARD_MSG_TRUNCATE_LAST_SHARD_FAILED,
         SHARD_MSG_UPDATE_FILE_SIZE_FAILED,
         SHARD_MSG_FOP_NOT_SUPPORTED,
-        SHARD_MSG_INVALID_FOP
+        SHARD_MSG_INVALID_FOP,
+        SHARD_MSG_MEMALLOC_FAILED
 );
 
 #endif /* !_SHARD_MESSAGES_H_ */
