@@ -15,6 +15,7 @@
 #include "xlator.h"
 #include "compat-errno.h"
 #include "shard-messages.h"
+#include "syncop.h"
 
 #define GF_SHARD_DIR ".shard"
 #define SHARD_MIN_BLOCK_SIZE  (4 * GF_UNIT_MB)
@@ -215,6 +216,7 @@ typedef struct shard_local {
         uint32_t gid;
         uint64_t block_size;
         uint64_t dst_block_size;
+        int32_t datasync;
         off_t offset;
         size_t total_size;
         size_t written_size;
@@ -256,6 +258,9 @@ typedef struct shard_local {
         } lock;
         inode_t *resolver_base_inode;
         gf_boolean_t first_lookup_done;
+        syncbarrier_t barrier;
+        gf_boolean_t lookup_shards_barriered;
+        gf_boolean_t unlink_shards_barriered;
 } shard_local_t;
 
 typedef struct shard_inode_ctx {
@@ -270,6 +275,15 @@ typedef struct shard_inode_ctx {
         uuid_t base_gfid;
         int block_num;
         gf_boolean_t refreshed;
+        struct list_head to_fsync_list;
+        int fsync_needed;
+        inode_t *inode;
+        int fsync_count;
+        inode_t *base_inode;
 } shard_inode_ctx_t;
 
+typedef enum {
+        SHARD_INTERNAL_DIR_DOT_SHARD = 1,
+} shard_internal_dir_type_t;
+
 #endif /* __SHARD_H__ */
