@@ -455,7 +455,6 @@ gotvolinfo:
                                 strcat (dup_volid, ".tcp");
                                 break;
                         default:
-                                ret = -1;
                                 break;
                         }
                 }
@@ -652,7 +651,7 @@ glusterd_create_missed_snap (glusterd_missed_snap_info *missed_snapinfo,
                 goto out;
         }
         strncpy (brickinfo->device_path, device,
-                 sizeof(brickinfo->device_path));
+                 sizeof(brickinfo->device_path) - 1);
 
         /* Update the backend file-system type of snap brick in
          * snap volinfo. */
@@ -869,7 +868,6 @@ __server_getspec (rpcsvc_request_t *req)
         char                 *brick_name             = NULL;
         char                 *volume                 = NULL;
         char                 *tmp                    = NULL;
-        int                   cookie                 = 0;
         rpc_transport_t      *trans                  = NULL;
         gf_getspec_req        args                   = {0,};
         gf_getspec_rsp        rsp                    = {0,};
@@ -999,8 +997,6 @@ fail:
 
         if (op_errno)
                 rsp.op_errno = gf_errno_to_error (op_errno);
-        if (cookie)
-                rsp.op_errno = cookie;
 
         if (!rsp.spec)
                 rsp.spec = strdup ("");
@@ -1025,7 +1021,6 @@ int32_t
 __server_event_notify (rpcsvc_request_t *req)
 {
         int32_t                 ret             = -1;
-        int32_t                 op_errno        =  0;
         gf_event_notify_req     args            = {0,};
         gf_event_notify_rsp     rsp             = {0,};
         dict_t                 *dict            = NULL;
@@ -1074,9 +1069,6 @@ __server_event_notify (rpcsvc_request_t *req)
 fail:
         rsp.op_ret   = ret;
 
-        if (op_errno)
-                rsp.op_errno = gf_errno_to_error (op_errno);
-
         if (need_rsp)
                 glusterd_submit_reply (req, &rsp, NULL, 0, NULL,
                                        (xdrproc_t)xdr_gf_event_notify_rsp);
@@ -1301,6 +1293,9 @@ out:
         if (rsp.hndsk.hndsk_val)
                 GF_FREE (rsp.hndsk.hndsk_val);
 
+        if (args_dict)
+                dict_unref (args_dict);
+
         return ret;
 }
 
@@ -1792,7 +1787,7 @@ gd_validate_peer_op_version (xlator_t *this, glusterd_peerinfo_t *peerinfo,
         int32_t          peer_min_op_version = 0;
         int32_t          peer_max_op_version = 0;
 
-        if (!dict && !this && !peerinfo)
+        if (!dict || !this || !peerinfo)
                 goto out;
 
         conf = this->private;
@@ -1823,8 +1818,9 @@ gd_validate_peer_op_version (xlator_t *this, glusterd_peerinfo_t *peerinfo,
 
         ret = 0;
 out:
-        gf_msg_debug (this->name , 0, "Peer %s %s", peerinfo->hostname,
-                ((ret < 0) ? "rejected" : "accepted"));
+        gf_msg_debug ((this ? this->name : "glusterd") , 0, "Peer %s %s",
+                      peerinfo->hostname, ((ret < 0) ? "rejected" : "accepted")
+                     );
         return ret;
 }
 
@@ -1891,7 +1887,7 @@ __glusterd_mgmt_hndsk_version_ack_cbk (struct rpc_req *req, struct iovec *iov,
         ret = default_notify (this, GF_EVENT_CHILD_UP, NULL);
 
         if (GD_MODE_ON == peerctx->args.mode) {
-                ret = glusterd_event_connected_inject (peerctx);
+                (void) glusterd_event_connected_inject (peerctx);
                 peerctx->args.req = NULL;
         } else if (GD_MODE_SWITCH_ON == peerctx->args.mode) {
                 peerctx->args.mode = GD_MODE_ON;
@@ -2286,7 +2282,7 @@ __glusterd_peer_dump_version_cbk (struct rpc_req *req, struct iovec *iov,
         ret = default_notify (this, GF_EVENT_CHILD_UP, NULL);
 
         if (GD_MODE_ON == peerctx->args.mode) {
-                ret = glusterd_event_connected_inject (peerctx);
+                (void) glusterd_event_connected_inject (peerctx);
                 peerctx->args.req = NULL;
         } else if (GD_MODE_SWITCH_ON == peerctx->args.mode) {
                 peerctx->args.mode = GD_MODE_ON;
