@@ -7,11 +7,6 @@
    later), or the GNU General Public License, version 2 (GPLv2), in all
    cases as published by the Free Software Foundation.
 */
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "common-utils.h"
 #include "cli1-xdr.h"
 #include "xdr-generic.h"
@@ -20,11 +15,13 @@
 #include "glusterd-store.h"
 #include "glusterd-utils.h"
 #include "glusterd-volgen.h"
+#include "glusterd-messages.h"
+#include "syscall.h"
 
 #include <signal.h>
 
 int
-glusterd_handle_log_rotate (rpcsvc_request_t *req)
+__glusterd_handle_log_rotate (rpcsvc_request_t *req)
 {
         int32_t                 ret     = -1;
         gf_cli_req              cli_req = {{0,}};
@@ -53,7 +50,8 @@ glusterd_handle_log_rotate (rpcsvc_request_t *req)
                                         cli_req.dict.dict_len,
                                         &dict);
                 if (ret < 0) {
-                        gf_log (this->name, GF_LOG_ERROR,
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                GD_MSG_DICT_UNSERIALIZE_FAIL,
                                 "failed to "
                                 "unserialize req-buffer to dictionary");
                         snprintf (msg, sizeof (msg), "Unable to decode the "
@@ -65,24 +63,23 @@ glusterd_handle_log_rotate (rpcsvc_request_t *req)
         ret = dict_get_str (dict, "volname", &volname);
         if (ret) {
                 snprintf (msg, sizeof (msg), "Failed to get volume name");
-                gf_log (this->name, GF_LOG_ERROR, "%s", msg);
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        GD_MSG_DICT_GET_FAILED, "%s", msg);
                 goto out;
         }
 
-        gf_log (this->name, GF_LOG_INFO, "Received log rotate req "
+        gf_msg (this->name, GF_LOG_INFO, 0,
+                GD_MSG_LOG_ROTATE_REQ_RECVD,
+                "Received log rotate req "
                 "for volume %s", volname);
 
         ret = dict_set_uint64 (dict, "rotate-key", (uint64_t)time (NULL));
         if (ret)
                 goto out;
 
-        ret = glusterd_op_begin (req, GD_OP_LOG_ROTATE, dict,
-                                 msg, sizeof (msg));
+        ret = glusterd_op_begin_synctask (req, GD_OP_LOG_ROTATE, dict);
 
 out:
-        glusterd_friend_sm ();
-        glusterd_op_sm ();
-
         if (ret) {
                 if (msg[0] == '\0')
                         snprintf (msg, sizeof (msg), "Operation failed");
@@ -94,6 +91,13 @@ out:
         return ret;
 }
 
+int
+glusterd_handle_log_rotate (rpcsvc_request_t *req)
+{
+        return glusterd_big_locked_handler (req,
+                                            __glusterd_handle_log_rotate);
+}
+
 /* op-sm */
 int
 glusterd_op_stage_log_rotate (dict_t *dict, char **op_errstr)
@@ -107,7 +111,8 @@ glusterd_op_stage_log_rotate (dict_t *dict, char **op_errstr)
 
         ret = dict_get_str (dict, "volname", &volname);
         if (ret) {
-                gf_log ("", GF_LOG_ERROR, "Unable to get volume name");
+                gf_msg ("glusterd", GF_LOG_ERROR, 0,
+                        GD_MSG_DICT_GET_FAILED, "Unable to get volume name");
                 goto out;
         }
 
@@ -116,7 +121,8 @@ glusterd_op_stage_log_rotate (dict_t *dict, char **op_errstr)
         if (!exists) {
                 snprintf (msg, sizeof (msg), "Volume %s does not exist",
                           volname);
-                gf_log ("", GF_LOG_ERROR, "%s", msg);
+                gf_msg ("glusterd", GF_LOG_ERROR, 0,
+                        GD_MSG_VOL_NOT_FOUND, "%s", msg);
                 *op_errstr = gf_strdup (msg);
                 ret = -1;
                 goto out;
@@ -125,7 +131,8 @@ glusterd_op_stage_log_rotate (dict_t *dict, char **op_errstr)
         if (_gf_false == glusterd_is_volume_started (volinfo)) {
                 snprintf (msg, sizeof (msg), "Volume %s needs to be started before"
                           " log rotate.", volname);
-                gf_log ("", GF_LOG_ERROR, "%s", msg);
+                gf_msg ("glusterd", GF_LOG_ERROR, 0,
+                        GD_MSG_VOL_NOT_STARTED, "%s", msg);
                 *op_errstr = gf_strdup (msg);
                 ret = -1;
                 goto out;
@@ -139,16 +146,18 @@ glusterd_op_stage_log_rotate (dict_t *dict, char **op_errstr)
                 goto out;
         }
 
-        ret = glusterd_volume_brickinfo_get_by_brick (brick, volinfo, NULL);
+        ret = glusterd_volume_brickinfo_get_by_brick (brick, volinfo, NULL,
+                                                      _gf_false);
         if (ret) {
                 snprintf (msg, sizeof (msg), "Incorrect brick %s "
                           "for volume %s", brick, volname);
-                gf_log ("", GF_LOG_ERROR, "%s", msg);
+                gf_msg ("glusterd", GF_LOG_ERROR, EINVAL,
+                        GD_MSG_INVALID_ENTRY, "%s", msg);
                 *op_errstr = gf_strdup (msg);
                 goto out;
         }
 out:
-        gf_log ("", GF_LOG_DEBUG, "Returning %d", ret);
+        gf_msg_debug ("glusterd", 0, "Returning %d", ret);
 
         return ret;
 }
@@ -164,7 +173,6 @@ glusterd_op_log_rotate (dict_t *dict)
         xlator_t             *this               = NULL;
         char                 *volname            = NULL;
         char                 *brick              = NULL;
-        char                  path[PATH_MAX]     = {0,};
         char                  logfile[PATH_MAX]  = {0,};
         char                  pidfile[PATH_MAX]  = {0,};
         FILE                 *file               = NULL;
@@ -180,13 +188,15 @@ glusterd_op_log_rotate (dict_t *dict)
 
         ret = dict_get_str (dict, "volname", &volname);
         if (ret) {
-                gf_log ("", GF_LOG_ERROR, "volname not found");
+                gf_msg ("glusterd", GF_LOG_ERROR, 0,
+                        GD_MSG_DICT_GET_FAILED, "volname not found");
                 goto out;
         }
 
         ret = dict_get_uint64 (dict, "rotate-key", &key);
         if (ret) {
-                gf_log ("", GF_LOG_ERROR, "rotate key not found");
+                gf_msg ("glusterd", GF_LOG_ERROR, 0,
+                        GD_MSG_DICT_GET_FAILED, "rotate key not found");
                 goto out;
         }
 
@@ -196,9 +206,11 @@ glusterd_op_log_rotate (dict_t *dict)
         if (ret)
                 goto cont;
 
-        ret = glusterd_brickinfo_new_from_brick (brick, &tmpbrkinfo);
+        ret = glusterd_brickinfo_new_from_brick (brick, &tmpbrkinfo,
+                                                 _gf_false, NULL);
         if (ret) {
-                gf_log ("glusterd", GF_LOG_ERROR,
+                gf_msg ("glusterd", GF_LOG_ERROR, 0,
+                        GD_MSG_BRICK_NOT_FOUND,
                         "cannot get brickinfo from brick");
                 goto out;
         }
@@ -209,8 +221,8 @@ cont:
                 goto out;
 
         ret = -1;
-        list_for_each_entry (brickinfo, &volinfo->bricks, brick_list) {
-                if (uuid_compare (brickinfo->uuid, MY_UUID))
+        cds_list_for_each_entry (brickinfo, &volinfo->bricks, brick_list) {
+                if (gf_uuid_compare (brickinfo->uuid, MY_UUID))
                         continue;
 
                 if (brick &&
@@ -220,13 +232,11 @@ cont:
 
                 valid_brick = 1;
 
-                GLUSTERD_GET_VOLUME_DIR (path, volinfo, priv);
-                GLUSTERD_GET_BRICK_PIDFILE (pidfile, path, brickinfo->hostname,
-                                            brickinfo->path);
-
+                GLUSTERD_GET_BRICK_PIDFILE (pidfile, volinfo, brickinfo, priv);
                 file = fopen (pidfile, "r+");
                 if (!file) {
-                        gf_log ("", GF_LOG_ERROR, "Unable to open pidfile: %s",
+                        gf_msg ("glusterd", GF_LOG_ERROR, errno,
+                                GD_MSG_FILE_OP_FAILED, "Unable to open pidfile: %s",
                                 pidfile);
                         ret = -1;
                         goto out;
@@ -234,7 +244,9 @@ cont:
 
                 ret = fscanf (file, "%d", &pid);
                 if (ret <= 0) {
-                        gf_log ("", GF_LOG_ERROR, "Unable to read pidfile: %s",
+                        fclose (file);
+                        gf_msg ("glusterd", GF_LOG_ERROR, errno,
+                                GD_MSG_FILE_OP_FAILED, "Unable to read pidfile: %s",
                                 pidfile);
                         ret = -1;
                         goto out;
@@ -245,13 +257,15 @@ cont:
                 snprintf (logfile, PATH_MAX, "%s.%"PRIu64,
                           brickinfo->logfile, key);
 
-                ret = rename (brickinfo->logfile, logfile);
+                ret = sys_rename (brickinfo->logfile, logfile);
                 if (ret)
-                        gf_log ("", GF_LOG_WARNING, "rename failed");
+                        gf_msg ("glusterd", GF_LOG_WARNING, errno,
+                                GD_MSG_FILE_OP_FAILED, "rename failed");
 
                 ret = kill (pid, SIGHUP);
                 if (ret) {
-                        gf_log ("", GF_LOG_ERROR, "Unable to SIGHUP to %d", pid);
+                        gf_msg ("glusterd", GF_LOG_ERROR, errno,
+                                GD_MSG_PID_KILL_FAIL, "Unable to SIGHUP to %d", pid);
                         goto out;
                 }
                 ret = 0;
