@@ -14,10 +14,8 @@
 #include "xlator.h"
 #include "iobuf.h"
 #include "call-stub.h"
-#include "stack.h"
 #include "nfs.h"
 #include "nfs-fops.h"
-#include "inode.h"
 #include "nfs-common.h"
 #include "nfs3-helpers.h"
 #include "nfs-mem-types.h"
@@ -46,7 +44,8 @@ nfs_fix_groups (xlator_t *this, call_stack_t *root)
         }
 
         /* RPC enforces the GF_AUTH_GLUSTERFS_MAX_GROUPS limit */
-        max_groups = GF_AUTH_GLUSTERFS_MAX_GROUPS(root->lk_owner.len);
+        max_groups = GF_AUTH_GLUSTERFS_MAX_GROUPS(root->lk_owner.len,
+                                                  AUTH_GLUSTERFS_v2);
 
 	agl = gid_cache_lookup(&priv->gid_cache, root->uid, 0, 0);
 	if (agl) {
@@ -330,7 +329,7 @@ dict_t *
 nfs_gfid_dict (inode_t *inode)
 {
         uuid_t  newgfid = {0, };
-        char    *dyngfid = NULL;
+        unsigned char *dyngfid = NULL;
         dict_t  *dictgfid = NULL;
         int     ret = -1;
         uuid_t  rootgfid = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1};
@@ -355,7 +354,7 @@ nfs_gfid_dict (inode_t *inode)
                 return (NULL);
         }
 
-        ret = dict_set_bin (dictgfid, "gfid-req", dyngfid, sizeof (uuid_t));
+        ret = dict_set_gfuuid (dictgfid, "gfid-req", dyngfid, false);
         if (ret < 0) {
                 GF_FREE (dyngfid);
                 dict_unref (dictgfid);
