@@ -8,15 +8,9 @@
   cases as published by the Free Software Foundation.
 */
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 
 #include "string.h"
 
-#include "inode.h"
 #include "nfs.h"
 #include "nfs-fops.h"
 #include "nfs-inodes.h"
