@@ -1099,7 +1099,7 @@ ioc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd,
         ioc_local_t *local         = NULL;
         uint32_t     weight        = 0;
         ioc_table_t *table         = NULL;
-        int32_t      op_errno      = -1;
+        int32_t      op_errno      = EINVAL;
 
         if (!this) {
                 goto out;
@@ -1411,10 +1411,8 @@ ioc_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         char        *path  = NULL;
         fd_t        *fd    = NULL;
 
-        if (frame) {
-                fd = frame->local;
-                frame->local = NULL;
-        }
+        fd = frame->local;
+        frame->local = NULL;
 
         if (op_ret <= 0)
                 goto unwind;
@@ -1666,6 +1664,9 @@ reconfigure (xlator_t *this, dict_t *options)
 
         ioc_table_lock (table);
         {
+                GF_OPTION_RECONF ("pass-through", this->pass_through, options,
+                                  bool, out);
+
                 GF_OPTION_RECONF ("cache-timeout", table->cache_timeout,
                                   options, int32, unlock);
 
@@ -1764,6 +1765,8 @@ init (xlator_t *this)
         table->xl = this;
         table->page_size = this->ctx->page_size;
 
+        GF_OPTION_INIT ("pass-through", this->pass_through, bool, out);
+
         GF_OPTION_INIT ("cache-size", table->cache_size, size_uint64, out);
 
         GF_OPTION_INIT ("cache-timeout", table->cache_timeout, int32, out);
@@ -2198,5 +2201,13 @@ struct volume_options options[] = {
           .op_version = {1},
           .flags = OPT_FLAG_CLIENT_OPT | OPT_FLAG_SETTABLE | OPT_FLAG_DOC
         },
+        { .key  = {"pass-through"},
+          .type = GF_OPTION_TYPE_BOOL,
+          .default_value = "false",
+          .op_version = {GD_OP_VERSION_4_1_0},
+          .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC | OPT_FLAG_CLIENT_OPT,
+          .tags = {"io-cache"},
+          .description = "Enable/Disable io cache translator"
+        },
         { .key = {NULL} },
 };
