@@ -27,8 +27,6 @@
 #include "client-messages.h"
 
 /* FIXME: Needs to be defined in a common file */
-#define CLIENT_CMD_CONNECT    "trusted.glusterfs.client-connect"
-#define CLIENT_CMD_DISCONNECT "trusted.glusterfs.client-disconnect"
 #define CLIENT_DUMP_LOCKS     "trusted.glusterfs.clientlk-dump"
 #define GF_MAX_SOCKET_WINDOW_SIZE  (1 * GF_UNIT_MB)
 #define GF_MIN_SOCKET_WINDOW_SIZE  (0)
@@ -237,6 +235,11 @@ typedef struct clnt_conf {
 
         gf_boolean_t           child_up; /* Set to true, when child is up, and
                                           * false, when child is down */
+
+        gf_boolean_t           can_log_disconnect; /* socket level connection is
+                                                    * up, disconnects can be
+                                                    * logged
+                                                    */
 } clnt_conf_t;
 
 typedef struct _client_fd_ctx {
