@@ -167,7 +167,7 @@ ret:
 
 
 int
-server_priv_to_dict (xlator_t *this, dict_t *dict)
+server_priv_to_dict (xlator_t *this, dict_t *dict, char *brickname)
 {
         server_conf_t   *conf = NULL;
         rpc_transport_t *xprt = NULL;
@@ -187,47 +187,55 @@ server_priv_to_dict (xlator_t *this, dict_t *dict)
         pthread_mutex_lock (&conf->mutex);
         {
                 list_for_each_entry (xprt, &conf->xprt_list, list) {
-                        peerinfo = &xprt->peerinfo;
-                        memset (key, 0, sizeof (key));
-                        snprintf (key, sizeof (key), "client%d.hostname",
-                                  count);
-                        ret = dict_set_str (dict, key, peerinfo->identifier);
-                        if (ret)
-                                goto unlock;
+                        if (!strcmp (brickname,
+                                     xprt->xl_private->bound_xl->name)) {
+                                peerinfo = &xprt->peerinfo;
+                                memset (key, 0, sizeof (key));
+                                snprintf (key, sizeof (key),
+                                          "client%d.hostname",
+                                          count);
+                                ret = dict_set_str (dict, key,
+                                                    peerinfo->identifier);
+                                if (ret)
+                                        goto unlock;
 
-                        memset (key, 0, sizeof (key));
-                        snprintf (key, sizeof (key), "client%d.bytesread",
-                                  count);
-                        ret = dict_set_uint64 (dict, key,
-                                               xprt->total_bytes_read);
-                        if (ret)
-                                goto unlock;
+                                memset (key, 0, sizeof (key));
+                                snprintf (key, sizeof (key),
+                                          "client%d.bytesread",
+                                          count);
+                                ret = dict_set_uint64 (dict, key,
+                                                xprt->total_bytes_read);
+                                if (ret)
+                                        goto unlock;
 
-                        memset (key, 0, sizeof (key));
-                        snprintf (key, sizeof (key), "client%d.byteswrite",
-                                  count);
-                        ret = dict_set_uint64 (dict, key,
-                                               xprt->total_bytes_write);
-                        if (ret)
-                                goto unlock;
+                                memset (key, 0, sizeof (key));
+                                snprintf (key, sizeof (key),
+                                          "client%d.byteswrite",
+                                          count);
+                                ret = dict_set_uint64 (dict, key,
+                                                xprt->total_bytes_write);
+                                if (ret)
+                                        goto unlock;
 
-                        memset (key, 0, sizeof (key));
-                        snprintf (key, sizeof (key), "client%d.opversion",
-                                  count);
-                        ret = dict_set_uint32 (dict, key,
-                                               peerinfo->max_op_version);
-                        if (ret)
-                                goto unlock;
+                                memset (key, 0, sizeof (key));
+                                snprintf (key, sizeof (key),
+                                          "client%d.opversion",
+                                          count);
+                                ret = dict_set_uint32 (dict, key,
+                                                peerinfo->max_op_version);
+                                if (ret)
+                                        goto unlock;
 
-                        memset (key, 0, sizeof (key));
-                        snprintf (key, sizeof (key), "client%d.name",
-                                  count);
-                        ret = dict_set_str (dict, key,
-                                               xprt->xl_private->client_name);
-                        if (ret)
-                                goto unlock;
+                                memset (key, 0, sizeof (key));
+                                snprintf (key, sizeof (key), "client%d.name",
+                                          count);
+                                ret = dict_set_str (dict, key,
+                                                xprt->xl_private->client_name);
+                                if (ret)
+                                        goto unlock;
 
-                        count++;
+                                count++;
+                        }
                 }
         }
 unlock:
@@ -1411,7 +1419,7 @@ server_notify (xlator_t *this, int32_t event, void *data, ...)
         case GF_EVENT_CHILD_UP:
         {
                 list_for_each_entry (tmp, &conf->child_status->status_list,
-                                                                 status_list) {
+                                     status_list) {
                         if (tmp->name == NULL)
                                 break;
                         if (strcmp (tmp->name, victim->name) == 0)
@@ -1426,7 +1434,7 @@ server_notify (xlator_t *this, int32_t event, void *data, ...)
                         tmp->name  = gf_strdup (victim->name);
                         tmp->child_up = _gf_true;
                         list_add_tail (&tmp->status_list,
-                                              &conf->child_status->status_list);
+                                       &conf->child_status->status_list);
                 }
                 ret = server_process_child_event (this, event, data,
                                                   GF_CBK_CHILD_UP);
@@ -1443,7 +1451,7 @@ server_notify (xlator_t *this, int32_t event, void *data, ...)
         case GF_EVENT_CHILD_DOWN:
         {
                 list_for_each_entry (tmp, &conf->child_status->status_list,
-                                                                  status_list) {
+                                     status_list) {
                         if (strcmp (tmp->name, victim->name) == 0) {
                                 tmp->child_up = _gf_false;
                                 break;
@@ -1506,7 +1514,7 @@ server_notify (xlator_t *this, int32_t event, void *data, ...)
                                                    trav_p = &(*trav_p)->next) {
                                         travxl = (*trav_p)->xlator;
                                         if (travxl &&
-                                                   strcmp (travxl->name, victim->name) == 0) {
+                                            strcmp (travxl->name, victim->name) == 0) {
                                                 victim_found = _gf_true;
                                                 break;
                                         }
@@ -1519,7 +1527,7 @@ server_notify (xlator_t *this, int32_t event, void *data, ...)
                                 (*trav_p) = (*trav_p)->next;
                         rpc_clnt_mgmt_pmap_signout (ctx, victim->name);
                         /* we need the protocol/server xlator here as 'this' */
-                        rpcsvc_autoscale_threads (ctx, -1, this);
+                        glusterfs_autoscale_threads (ctx, -1, this);
                         default_notify (victim, GF_EVENT_CLEANUP, data);
                 }
                 break;
