@@ -2148,7 +2148,7 @@ bd_rchecksum (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset,
         char           *buf             = NULL;
         int32_t         weak_checksum   = 0;
         bd_fd_t        *bd_fd           = NULL;
-        unsigned char   strong_checksum[MD5_DIGEST_LENGTH] = {0};
+        unsigned char   strong_checksum[SHA256_DIGEST_LENGTH] = {0};
 
         VALIDATE_OR_GOTO (frame, out);
         VALIDATE_OR_GOTO (this, out);
@@ -2162,8 +2162,6 @@ bd_rchecksum (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset,
                 return 0;
         }
 
-        memset (strong_checksum, 0, MD5_DIGEST_LENGTH);
-
         alloc_buf = page_aligned_alloc (len, &buf);
         if (!alloc_buf) {
                 op_errno = ENOMEM;
