@@ -63,11 +63,12 @@
         }                                                               \
         if (LOC_IS_DIR (loc) && LOC_HAS_ABSPATH (loc)) {                \
                 MAKE_REAL_PATH (rpath, this, (loc)->path);              \
-                op_ret = posix_pstat (this, (loc)->gfid, rpath, iatt_p); \
+                op_ret = posix_pstat (this, (loc)->inode, (loc)->gfid,  \
+                                      rpath, iatt_p, _gf_false);        \
                 break;                                                  \
         }                                                               \
         errno = 0;                                                      \
-        op_ret = posix_istat (this, loc->gfid, NULL, iatt_p);           \
+        op_ret = posix_istat (this, loc->inode, loc->gfid, NULL, iatt_p); \
         if (errno != ELOOP) {                                           \
                 MAKE_HANDLE_PATH (rpath, this, (loc)->gfid, NULL);      \
                 if (!rpath) {                                           \
